# -*- coding: utf-8 -*-
"""Helper functions for controller operation.

URL definition and browser redirection are defined here.

"""
import re
import warnings
from webob.exc import status_map

import tg

from tg._compat import string_type, url_encode, unicode_text, bytes_
from tg.exceptions import HTTPFound
from tg.request_local import request, response, Response
from tg.configuration.utils import TGConfigError


def _smart_str(s):
    """
    Returns a bytestring version of 's', encoded as specified in 'encoding'.

    If strings_only is True, don't convert (some) non-string-like objects.

    This function was borrowed from Django.

    """
    if not isinstance(s, string_type):
        try:
            return bytes_(s)
        except UnicodeEncodeError:
            if isinstance(s, Exception):
                # An Exception subclass containing non-ASCII data that doesn't
                # know how to print itself properly. We shouldn't raise a
                # further exception.
                return ' '.join([_smart_str(arg).decode('utf-8') for arg in s.args]).encode('utf-8', 'strict')
            return unicode_text(s).encode('utf-8', 'strict')
    elif isinstance(s, unicode_text):
        return s.encode('utf-8', 'strict')
    else:
        return s


def _generate_smart_str(params):
    for key, value in params.items():
        if value is None:
            continue
        if isinstance(value, (list, tuple)):
            for item in value:
                yield _smart_str(key), _smart_str(item)
        else:
            yield _smart_str(key), _smart_str(value)


def _urlencode(params):
    """
    A version of Python's urllib.urlencode() function that can operate on
    unicode strings. The parameters are first case to UTF-8 encoded strings and
    then encoded as per normal.
    """
    return url_encode([i for i in _generate_smart_str(params)])


def _build_url(environ, base_url='/', params=None):
    if base_url.startswith('/'):
        base_url = environ['SCRIPT_NAME'] + base_url

    if params:
        return '?'.join((base_url, _urlencode(params)))

    return base_url


def url(base_url='/', params=None, qualified=False, scheme=None):
    """Generate an absolute URL that's specific to this application.

    The URL function takes a string (base_url) and, appends the
    SCRIPT_NAME and adds parameters for all of the
    parameters passed into the params dict.

    ``scheme`` can be passed in case of a ``qualified`` url to
    create an url with the given scheme.

    """
    if not isinstance(base_url, string_type) and hasattr(base_url, '__iter__'):
        base_url = '/'.join(base_url)

    req = tg.request._current_obj()
    base_url = _build_url(req.environ, base_url, params)
    if qualified:
        base_url = req.host_url + base_url
        if scheme is not None:
            base_url = scheme + base_url[len(req.scheme):]

    return base_url


class LazyUrl(object):
    """
    Wraps tg.url in an object that enforces evaluation of the url
    only when you try to display it as a string.
    """

    def __init__(self, base_url, params=None, **kwargs):
        self.base_url = base_url
        self.params = params
        self.kwargs = kwargs
        self._decoded = None

    @property
    def _id(self):
        if self._decoded == None:
            self._decoded = url(self.base_url, params=self.params, **self.kwargs)
        return self._decoded

    @property
    def id(self):
        return self._id

    def __repr__(self):
        return self._id

    def __html__(self):
        return str(self)

    def __str__(self):
        return str(self._id)

    def encode(self, *args, **kw):
        return self._id.encode(*args, **kw)

    def __add__(self, other):
        return self._id + other

    def __radd__(self, other):
        return other + self._id

    def startswith(self, *args, **kw):
        return self._id.startswith(*args, **kw)

    def format(self, *args, **kwargs):
        return self._id.format(*args, **kwargs)

    def __json__(self):
        return str(self)


def lurl(base_url=None, params=None, **kwargs):
    """
    Like tg.url but is lazily evaluated.

    This is useful when creating global variables as no
    request is in place.

    As without a request it wouldn't be possible
    to correctly calculate the url using the SCRIPT_NAME
    this demands the url resolution to when it is
    displayed for the first time.
    """
    return LazyUrl(base_url, params, **kwargs)


def redirect(base_url='/', params=None, redirect_with=HTTPFound, scheme=None, **kwargs):
    """Generate an HTTP redirect.

    The function raises an exception internally,
    which is handled by the framework. The URL may be either absolute (e.g.
    http://example.com or /myfile.html) or relative. Relative URLs are
    automatically converted to absolute URLs. Parameters may be specified,
    which are appended to the URL. This causes an external redirect via the
    browser; if the request is POST, the browser will issue GET for the
    second request.
    """

    if params is None:
        params = {}

    if kwargs:
        warnings.warn('passing URL parameters by name is deprecated, please use params argument',
                      DeprecationWarning)
        params = params.copy()
        params.update(kwargs)

    new_url = url(base_url, params=params, scheme=scheme)
    raise redirect_with(location=new_url)


IF_NONE_MATCH = re.compile('(?:W/)?(?:"([^"]*)",?\s*)')
def etag_cache(key=None):
    """Use the HTTP Entity Tag cache for Browser side caching

    If a "If-None-Match" header is found, and equivilant to ``key``,
    then a ``304`` HTTP message will be returned with the ETag to tell
    the browser that it should use its current cache of the page.

    Otherwise, the ETag header will be added to the response headers.
    """
    if_none_matches = IF_NONE_MATCH.findall(tg.request.environ.get('HTTP_IF_NONE_MATCH', ''))
    response = tg.response._current_obj()
    response.headers['ETag'] = '"%s"' % key
    if str(key) in if_none_matches:
        response.headers.pop('Content-Type', None)
        response.headers.pop('Cache-Control', None)
        response.headers.pop('Pragma', None)
        raise status_map[304]()


def abort(status_code=None, detail="", headers=None, comment=None,
          passthrough=False, error_handler=False):
    """Aborts the request immediately by returning an HTTP exception

    In the event that the status_code is a 300 series error, the detail
    attribute will be used as the Location header should one not be
    specified in the headers attribute.

    **passthrough**
        When ``True`` instead of displaying the custom error
        document for errors or the authentication page for
        failed authorizations the response will just pass
        through as is.

        Set to ``"json"`` to send out the response body in
        JSON format.

    **error_handler**
        When ``True`` instead of immediately abort the request
        it will create a callable that can be used as ``@validate``
        error_handler.

        A common case is ``abort(404, error_handler=True)`` as
        ``error_handler`` for validation that retrieves objects
        from database::

            from formencode.validators import Wrapper

            @validate({'team': Wrapper(to_python=lambda value:
                                        Group.query.find({'group_name': value}).one())},
                      error_handler=abort(404, error_handler=True))
            def view_team(self, team):
                return dict(team=team)

    """
    exc = status_map[status_code](detail=detail, headers=headers,
                                  comment=comment)

    if passthrough == 'json':
        exc.content_type = 'application/json'
        exc.charset = 'utf-8'
        exc.body = tg.json_encode(dict(status=status_code,
                                       detail=str(exc))).encode('utf-8')

    def _abortion(*args, **kwargs):
        if passthrough:
            tg.request.environ['tg.status_code_redirect'] = False
            tg.request.environ['tg.skip_auth_challenge'] = True
        raise exc

    if error_handler is False:
        return _abortion()
    else:
        return _abortion


def use_wsgi_app(wsgi_app):
    return tg.request.get_response(wsgi_app)


def auth_force_login(user_name):
    """Forces user login if authentication is enabled.

    As TurboGears identifies users by ``user_name`` the passed parameter should
    be anything your application declares being the ``user_name`` field in models.

    """
    req = request._current_obj()
    resp = response._current_obj()

    api = req.environ.get('repoze.who.api')
    if api:
        authentication_plugins = req.environ['repoze.who.plugins']
        try:
            identifier = authentication_plugins['main_identifier']
        except KeyError:
            raise TGConfigError('No repoze.who plugin registered as "main_identifier"')

        resp.headers.extend(api.remember({
            'repoze.who.userid': user_name,
            'identifier': identifier
        }))


def auth_force_logout():
    """Forces user logout if authentication is enabled."""
    req = request._current_obj()
    resp = response._current_obj()

    api = req.environ.get('repoze.who.api')
    if api:
        resp.headers.extend(api.forget())


def validation_errors_response(*args, **kwargs):
    """Returns a :class:`.Response` object with validation errors.

    The response will be created with a *412 Precondition Failed*
    status code and errors are reported in JSON format as response body.

    Typical usage is as ``error_handler`` for JSON based api::

        @expose('json')
        @validate({'display_name': validators.NotEmpty(),
                   'group_name': validators.NotEmpty()},
                  error_handler=validation_errors_response)
        def post(self, **params):
            group = Group(**params)
            return dict(group=group)

    """
    req = request._current_obj()
    errors = dict(
        (unicode_text(key), unicode_text(error)) for key, error in req.validation.errors.items()
    )
    values = req.validation.values
    try:
        return Response(status=412, json_body={'errors': errors,
                                               'values': values})
    except TypeError:
        # values cannot be encoded to JSON, this might happen after
        # validation passed and validators converted them to complex objects.
        # In this case use request params, instead of controller params.
        return Response(status=412, json_body={'errors': errors,
                                               'values': req.args_params})

__all__ = ['url', 'lurl', 'redirect', 'etag_cache', 'abort', 'auth_force_logout',
           'auth_force_login', 'validation_errors_response', 'use_wsgi_app']



from zope.interface import implementer
from repoze.who.interfaces import IAuthenticator


class TGAuthMetadata(object):
    """
    Provides a way to lookup for user, groups and permissions
    given the current identity. This has to be specialized
    for each storage backend.

    By default it returns empty lists for groups and permissions
    and None for the user.
    """
    def get_user(self, identity, userid):
        return None

    def get_groups(self, identity, userid):
        return []

    def get_permissions(self, identity, userid):
        return []


@implementer(IAuthenticator)
class _AuthMetadataAuthenticator(object):
    def __init__(self, tgmdprovider, using_password):
        self.tgmdprovider = tgmdprovider
        self.using_password = using_password

    # IAuthenticator
    def authenticate(self, environ, identity):
        if self.using_password and not ('login' in identity and 'password' in identity):
            return None
        return self.tgmdprovider.authenticate(environ, identity)


def create_default_authenticator(authmetadata,
                                 using_password=True, translations=None,
                                 user_class=None, dbsession=None,
                                 **kept_params):
    auth = _AuthMetadataAuthenticator(authmetadata, using_password)
    return kept_params, auth



from nose.tools import raises
from tg._compat import im_self
from tg.configuration.milestones import _ConfigMilestoneTracker, config_ready
from tg.configuration.utils import GlobalConfigurable, TGConfigError


class Action:
    called = 0
    def __call__(self):
        self.called += 1


class TestMilestones(object):
    def setup(self):
        self.milestone = _ConfigMilestoneTracker('test_milestone')

    def test_multiple_registration(self):
        a = Action()
        self.milestone.register(a)
        self.milestone.register(a)
        self.milestone.register(a)

        self.milestone.reach()
        assert a.called == 1

    def test_register_after_reach(self):
        a = Action()

        self.milestone.reach()
        self.milestone.register(a)
        assert a.called == 1

    def test_call_all(self):
        a = Action()
        a2 = Action()
        a3 = Action()

        self.milestone.register(a)
        self.milestone.register(a2)
        self.milestone.register(a3)

        self.milestone.reach()
        assert a.called == a2.called == a3.called == 1

    def test_register_func_unique(self):
        called = []
        def f():
            called.append(True)

        self.milestone.register(f)
        self.milestone.register(f)

        self.milestone.reach()
        assert len(called) == 1


class TemporaryGlobalConfigurable(GlobalConfigurable):
    pass

class TestGlobalConfigurable(object):
    def setup(self):
        config_ready._reset()

    def teardown(self):
        for action in config_ready._keep_on_reset[:]:
            default_object = im_self(action)
            if default_object and isinstance(default_object, TemporaryGlobalConfigurable):
                config_ready._keep_on_reset.remove(action)

        config_ready._reset()

    @raises(NotImplementedError)
    def test_requires_configure_implementation(self):
        class NoConfig(TemporaryGlobalConfigurable):
            CONFIG_NAMESPACE = 'fake.'

        default_object = NoConfig.create_global()
        config_ready.reach()

    @raises(TGConfigError)
    def test_requires_namespace(self):
        class NoNameSpace(TemporaryGlobalConfigurable):
            def configure(self, **options):
                return options

        default_object = NoNameSpace.create_global()
        config_ready.reach()

    def test_gets_configured_on_config_ready(self):
        class Configurable(TemporaryGlobalConfigurable):
            CONFIG_NAMESPACE = 'fake.'
            _CONFIGURED = []

            def configure(self, **options):
                self._CONFIGURED.append(True)

        default_object = Configurable.create_global()
        config_ready.reach()

        assert len(Configurable._CONFIGURED) == 1

    def test_keeps_on_milestone_reset(self):
        class Configurable(TemporaryGlobalConfigurable):
            CONFIG_NAMESPACE = 'fake.'
            _CONFIGURED = []

            def configure(self, **options):
                self._CONFIGURED.append(True)

        default_object = Configurable.create_global()
        config_ready.reach()

        config_ready._reset()
        config_ready.reach()

        assert len(Configurable._CONFIGURED) == 2


# -*- coding: utf-8 -*-
from repoze.who.plugins.sa import SQLAlchemyAuthenticatorPlugin

def create_default_authenticator(user_class, dbsession, translations=None, **unused):
    sqlauth = SQLAlchemyAuthenticatorPlugin(user_class, dbsession)
    if translations is not None:
        sqlauth.translations.update(translations)
    return unused, sqlauth



"""Caching decorator, took as is from pylons"""
import tg, inspect, time
from tg.support.converters import asbool
from tg.support import NoDefault, EmptyContext
from tg._compat import im_func, im_class
from functools import wraps


class cached_property(object):
    """
    Works like python @property but the decorated function only gets
    executed once, successive accesses to the property will just
    return the value previously stored into the object.

    The ``@cached_property`` decorator can be executed within a
    provided context, for example to make the cached property
    thread safe a Lock can be provided::

        from threading import Lock
        from tg.caching import cached_property

        class MyClass(object):
            @cached_property
            def my_property(self):
                return 'Value!'
            my_property.context = Lock()

    """
    def __init__(self, func):
        self.__name__ = func.__name__
        self.__module__ = func.__module__
        self.__doc__ = func.__doc__
        self.func = func
        self.context = EmptyContext()

    def _get_value(self, obj):
        value = obj.__dict__.get(self.__name__, NoDefault)
        if value is NoDefault:
            value = self.func(obj)
            obj.__dict__[self.__name__] = value
        return value

    def __get__(self, obj, type=None):
        if obj is None:
            return self

        with self.context:
            return self._get_value(obj)


def _cached_call(func, args, kwargs, namespace, cache_key,
                 expire="never", type=None, starttime=None,
                 cache_headers=('content-type', 'content-length'),
                 cache_response=True, cache_extra_args=None):
    """
    Optional arguments:

    ``key_func``
        Function used to genereate the cache key, the function name
        and class will be used as the base for the cache key. If None
        the ``func`` itself will be used. It's usually handy when
        creating caches for decorated functions, for which we want the
        cache key to be generated on the decorated function and not on
        the decorator.
    ``key_dict``
        Arguments used to generate the cache key, only the arguments
        listed into this dictionary will be used to generate the
        cache key together with the key_func.
    ``expire``
        Time in seconds before cache expires, or the string "never".
        Defaults to "never"
    ``type``
        Type of cache to use: dbm, memory, file, memcached, or None for
        Beaker's default
    ``cache_headers``
        A tuple of header names indicating response headers that
        will also be cached.
    ``invalidate_on_startup``
        If True, the cache will be invalidated each time the application
        starts or is restarted.
    ``cache_response``
        Determines whether the response at the time beaker_cache is used
        should be cached or not, defaults to True.

        .. note::
            When cache_response is set to False, the cache_headers
            argument is ignored as none of the response is cached.

    If cache.enabled is set to False in the .ini file, then cache is
    disabled globally.
    """

    tg_locals = tg.request.environ['tg.locals']
    enabled = asbool(tg_locals.config.get("cache.enabled", True))

    if not enabled:
        return func(*args, **kwargs)

    cache_extra_args = cache_extra_args or {}

    if type:
        cache_extra_args['type'] = type

    cache_obj = getattr(tg_locals, 'cache', None)
    if not cache_obj:  # pragma: no cover
        raise Exception('TurboGears Cache object not found, ensure cache.enabled=True')

    my_cache = cache_obj.get_cache(namespace, **cache_extra_args)

    if expire == "never":
        cache_expire = None
    else:
        cache_expire = expire

    def create_func():
        result = func(*args, **kwargs)
        glob_response = tg_locals.response
        headers = glob_response.headerlist
        status = glob_response.status
        full_response = dict(headers=headers, status=status,
                             cookies=None, content=result)
        return full_response

    response = my_cache.get_value(cache_key,
                                  createfunc=create_func,
                                  expiretime=cache_expire,
                                  starttime=starttime)
    if cache_response:
        glob_response = tg_locals.response
        glob_response.headerlist = [header for header in response['headers']
                                    if header[0].lower() in cache_headers]
        glob_response.status = response['status']

    return response['content']


def beaker_cache(key="cache_default", expire="never", type=None,
                 query_args=False,
                 cache_headers=('content-type', 'content-length'),
                 invalidate_on_startup=False,
                 cache_response=True, **b_kwargs):
    """Cache decorator utilizing Beaker. Caches a
    function that returns a pickle-able object as a result.

    Optional arguments:

    ``key``
        None - No variable key, uses function name as key
        "cache_default" - Uses all function arguments as the key
        string - Use kwargs[key] as key
        list - Use [kwargs[k] for k in list] as key
    ``expire``
        Time in seconds before cache expires, or the string "never".
        Defaults to "never"
    ``type``
        Type of cache to use: dbm, memory, file, memcached, or None for
        Beaker's default
    ``query_args``
        Uses the query arguments as the key, defaults to False
    ``cache_headers``
        A tuple of header names indicating response headers that
        will also be cached.
    ``invalidate_on_startup``
        If True, the cache will be invalidated each time the application
        starts or is restarted.
    ``cache_response``
        Determines whether the response at the time beaker_cache is used
        should be cached or not, defaults to True.

        .. note::
            When cache_response is set to False, the cache_headers
            argument is ignored as none of the response is cached.

    If cache.enabled is set to False in the .ini file, then cache is
    disabled globally.

    """
    if invalidate_on_startup:
        starttime = time.time()
    else:
        starttime = None
    cache_headers = set(cache_headers)

    def beaker_cache_decorate(func):
        @wraps(func)
        def beaker_cached_call(*args, **kwargs):
            if key:
                key_dict = kwargs.copy()
                key_dict.update(_make_dict_from_args(func, args, kwargs))
                if query_args:
                    key_dict.update(tg.request.GET.mixed())

                if key != 'cache_default':
                    if isinstance(key, list):
                        key_dict = dict((k, key_dict[k]) for k in key)
                    else:
                        key_dict = {key: key_dict[key]}
            else:
                key_dict = None

            self = None
            if args:
                self = args[0]
            namespace, cache_key = create_cache_key(func, key_dict, self)

            return _cached_call(func, args, kwargs, namespace, cache_key,
                                expire, type, starttime,
                                cache_headers, cache_response,
                                b_kwargs)

        return beaker_cached_call

    return beaker_cache_decorate


def create_cache_key(func, key_dict=None, self=None):
    """Get a cache namespace and key used by the beaker_cache decorator.

    Example::
        from tg import cache
        from tg.caching import create_cache_key
        namespace, key = create_cache_key(MyController.some_method)
        cache.get_cache(namespace).remove(key)

    """
    kls = None
    imfunc = im_func(func)
    if imfunc:
        kls = im_class(func)
        func = imfunc
        cache_key = func.__name__
    else:
        cache_key = func.__name__
    if key_dict:
        cache_key += " " + " ".join("%s=%s" % (k, v)
                                    for k, v in key_dict.items())

    if not kls and self:
        kls = getattr(self, '__class__', None)

    if kls:
        return '%s.%s' % (kls.__module__, kls.__name__), cache_key
    else:
        return func.__module__, cache_key


def _make_dict_from_args(func, args, kwargs):
    """Inspects function for name of args"""
    args_keys = {}
    for i, arg in enumerate(inspect.getargspec(func)[0]):
        if arg != "self":
            try:
                args_keys[arg] = args[i]
            except IndexError:
                args_keys[arg] = kwargs[arg]
    return args_keys



# -*- coding: utf-8 -*-
"""
Built-in predicate checkers.

This is mostly took from repoze.what.precidates

This is module provides the predicate checkers that were present in the
original "identity" framework of TurboGears 1, plus others.

"""

from __future__ import unicode_literals
from tg import request
from tg._compat import unicode_text

__all__ = ['Predicate', 'CompoundPredicate', 'All', 'Any',
           'has_all_permissions', 'has_any_permission', 'has_permission',
           'in_all_groups', 'in_any_group', 'in_group', 'is_user',
           'is_anonymous', 'not_anonymous', 'NotAuthorizedError']

try: #pragma: no cover
    # If repoze.what is available use repoze.what Predicate and
    # NotAuthorizedError adding booleanization support to the
    # predicates
    from repoze.what.predicates import NotAuthorizedError, Predicate
    Predicate.__nonzero__ = lambda self: self.is_met(request.environ)
except ImportError:
    class NotAuthorizedError(Exception):
        pass

    class Predicate(object):
        def __init__(self, msg=None):
            if msg:
                self.message = msg

        def evaluate(self, environ, credentials):
            raise NotImplementedError

        def unmet(self, msg=None, **placeholders):
            """
            Raise an exception because this predicate is not met.

            :param msg: The error message to be used; overrides the predicate's
                default one.
            :type msg: str
            :raises NotAuthorizedError: If the predicate is not met.

            ``placeholders`` represent the placeholders for the predicate message.
            The predicate's attributes will also be taken into account while
            creating the message with its placeholders.
            """
            if msg:
                message = msg
            else:
                message = self.message

            # This enforces lazy strings resolution (lazy translation for example)
            message = unicode_text(message)

            # Include the predicate attributes in the placeholders:
            all_placeholders = self.__dict__.copy()
            all_placeholders.update(placeholders)

            raise NotAuthorizedError(message % all_placeholders)

        def check_authorization(self, environ):
            """
            Evaluate the predicate and raise an exception if it's not met.

            :param environ: The WSGI environment.
            :raise NotAuthorizedError: If it the predicate is not met.
            """
            credentials = environ.get('repoze.what.credentials', {})
            try:
                self.evaluate(environ, credentials)
            except NotAuthorizedError:
                raise

        def is_met(self, environ):
            """
            Find whether the predicate is met or not.

            :param environ: The WSGI environment.
            :return: Whether the predicate is met or not.
            :rtype: bool
            """
            credentials = environ.get('repoze.what.credentials', {})
            try:
                self.evaluate(environ, credentials)
                return True
            except NotAuthorizedError:
                return False

        def __nonzero__(self):
            return self.is_met(request.environ)
        __bool__ = __nonzero__

class CompoundPredicate(Predicate):
    """A predicate composed of other predicates."""

    def __init__(self, *predicates, **kwargs):
        super(CompoundPredicate, self).__init__(**kwargs)
        self.predicates = predicates


class Not(Predicate):
    """
    Negate the specified predicate.

    :param predicate: The predicate to be negated.

    Example::

        # The user *must* be anonymous:
        p = Not(not_anonymous())

    """
    message = "The condition must not be met"

    def __init__(self, predicate, **kwargs):
        super(Not, self).__init__(**kwargs)
        self.predicate = predicate

    def evaluate(self, environ, credentials):
        try:
            self.predicate.evaluate(environ, credentials)
        except NotAuthorizedError:
            return
        self.unmet()


class All(CompoundPredicate):
    """
    Check that all of the specified predicates are met.

    :param predicates: All of the predicates that must be met.

    Example::

        # Grant access if the current month is July and the user belongs to
        # the human resources group.
        p = All(is_month(7), in_group('hr'))

    """

    def evaluate(self, environ, credentials):
        """
        Evaluate all the predicates it contains.

        :param environ: The WSGI environment.
        :param credentials: The :mod:`repoze.what` ``credentials``.
        :raises NotAuthorizedError: If one of the predicates is not met.

        """
        for p in self.predicates:
            p.evaluate(environ, credentials)


class Any(CompoundPredicate):
    """
    Check that at least one of the specified predicates is met.

    :param predicates: Any of the predicates that must be met.

    Example::

        # Grant access if the currest user is Richard Stallman or Linus
        # Torvalds.
        p = Any(is_user('rms'), is_user('linus'))

    """
    message = "At least one of the following predicates must be met: %(failed_predicates)s"

    def evaluate(self, environ, credentials):
        """
        Evaluate all the predicates it contains.

        :param environ: The WSGI environment.
        :param credentials: The :mod:`repoze.what` ``credentials``.
        :raises NotAuthorizedError: If none of the predicates is met.

        """
        errors = []
        for p in self.predicates:
            try:
                p.evaluate(environ, credentials)
                return
            except NotAuthorizedError as exc:
                errors.append(unicode_text(exc))
        failed_predicates = ', '.join(errors)
        self.unmet(failed_predicates=failed_predicates)


class is_user(Predicate):
    """
    Check that the authenticated user's username is the specified one.

    :param user_name: The required user name.
    :type user_name: str

    Example::

        p = is_user('linus')

    """

    message = 'The current user must be "%(user_name)s"'

    def __init__(self, user_name, **kwargs):
        super(is_user, self).__init__(**kwargs)
        self.user_name = user_name

    def evaluate(self, environ, credentials):
        if credentials and\
           self.user_name == credentials.get('repoze.what.userid'):
            return
        self.unmet()


class in_group(Predicate):
    """
    Check that the user belongs to the specified group.

    :param group_name: The name of the group to which the user must belong.
    :type group_name: str

    Example::

        p = in_group('customers')

    """

    message = 'The current user must belong to the group "%(group_name)s"'

    def __init__(self, group_name, **kwargs):
        super(in_group, self).__init__(**kwargs)
        self.group_name = group_name

    def evaluate(self, environ, credentials):
        if credentials and self.group_name in credentials.get('groups'):
            return
        self.unmet()


class in_all_groups(All):
    """
    Check that the user belongs to all of the specified groups.

    :param groups: The name of all the groups the user must belong to.

    Example::

        p = in_all_groups('developers', 'designers')

    """


    def __init__(self, *groups, **kwargs):
        group_predicates = [in_group(g) for g in groups]
        super(in_all_groups,self).__init__(*group_predicates, **kwargs)


class in_any_group(Any):
    """
    Check that the user belongs to at least one of the specified groups.

    :param groups: The name of any of the groups the user may belong to.

    Example::

        p = in_any_group('directors', 'hr')

    """

    message = "The member must belong to at least one of the following groups: %(group_list)s"

    def __init__(self, *groups, **kwargs):
        self.group_list = ", ".join(groups)
        group_predicates = [in_group(g) for g in groups]
        super(in_any_group,self).__init__(*group_predicates, **kwargs)


class is_anonymous(Predicate):
    """
    Check that the current user is anonymous.

    Example::

        # The user must be anonymous!
        p = is_anonymous()

    .. versionadded:: 1.0.7

    """

    message = "The current user must be anonymous"

    def evaluate(self, environ, credentials):
        if credentials:
            self.unmet()


class not_anonymous(Predicate):
    """
    Check that the current user has been authenticated.

    Example::

        # The user must have been authenticated!
        p = not_anonymous()

    """

    message = "The current user must have been authenticated"

    def evaluate(self, environ, credentials):
        if not credentials:
            self.unmet()


class has_permission(Predicate):
    """
    Check that the current user has the specified permission.

    :param permission_name: The name of the permission that must be granted to
        the user.

    Example::

        p = has_permission('hire')

    """
    message = 'The user must have the "%(permission_name)s" permission'

    def __init__(self, permission_name, **kwargs):
        super(has_permission, self).__init__(**kwargs)
        self.permission_name = permission_name

    def evaluate(self, environ, credentials):
        if credentials and\
           self.permission_name in credentials.get('permissions'):
            return
        self.unmet()


class has_all_permissions(All):
    """
    Check that the current user has been granted all of the specified
    permissions.

    :param permissions: The names of all the permissions that must be
        granted to the user.

    Example::

        p = has_all_permissions('view-users', 'edit-users')

    """

    def __init__(self, *permissions, **kwargs):
        permission_predicates = [has_permission(p) for p in permissions]
        super(has_all_permissions, self).__init__(*permission_predicates,
            **kwargs)


class has_any_permission(Any):
    """
    Check that the user has at least one of the specified permissions.

    :param permissions: The names of any of the permissions that have to be
        granted to the user.

    Example::

        p = has_any_permission('manage-users', 'edit-users')

    """

    message = "The user must have at least one of the following permissions: %(permission_list)s"

    def __init__(self, *permissions, **kwargs):
        self.permission_list = ", ".join(permissions)
        permission_predicates = [has_permission(p) for p in permissions]
        super(has_any_permission,self).__init__(*permission_predicates,
            **kwargs)


import logging
from tg.configuration.utils import coerce_config
from tg.support.converters import asbool, asint
from .base import ApplicationWrapper

log = logging.getLogger(__name__)


class MingApplicationWrapper(ApplicationWrapper):
    """Automatically flushes the Ming ODMSession.

    In case an exception raised during excution it won't flush the session and it will
    instead close it throwing away any change.

    Supported options which can be provided by config are:

        - ``ming.autoflush``: Whenever to flush session at end of request if no exceptions happened.

    """
    def __init__(self, handler, config):
        super(MingApplicationWrapper, self).__init__(handler, config)

        options = {
            'autoflush': False,
        }
        options.update(coerce_config(config, 'ming.',  {
            'autoflush': asbool,
        }))

        self.ThreadLocalODMSession = None
        self.enabled = options['autoflush']

        if self.enabled:
            try:
                from ming.odm import ThreadLocalODMSession
                self.ThreadLocalODMSession = ThreadLocalODMSession
            except ImportError:  # pragma: no cover
                log.exception('Unable to Enable Ming Application Wrapper')
                self.enabled = False

        log.debug('MingSessionFlush enabled: %s -> %s',
                  self.enabled, options)

    @property
    def injected(self):
        return self.enabled

    def __call__(self, controller, environ, context):
        session = self.ThreadLocalODMSession

        try:
            resp = self.next_handler(controller, environ, context)
        except:
            session.close_all()
            raise

        log.debug('MingSessionFlush flushing changes...')
        session.flush_all()
        return resp



"""
New TurboGears2 identification, authentication and authorization setup.

This aims to provide an easier way to setup auth layer in TurboGears2
and removes the dependency from repoze.what.
"""
import sys, logging, re
from tg.support.converters import asbool
from zope.interface import implementer
from repoze.who.middleware import PluggableAuthenticationMiddleware
from repoze.who.interfaces import IIdentifier, IAuthenticator, IChallenger
from repoze.who.classifiers import default_challenge_decider, default_request_classifier
from repoze.who.config import _LEVELS
from webob.exc import HTTPUnauthorized

log = logging.getLogger(__name__)

@implementer(IIdentifier, IAuthenticator, IChallenger)
class _AuthenticationForgerPlugin(object):
    """
    Took from repoze.who_testutil.
    This is meant for internal use only to setup tests with fake
    authentication. It is a repoze.who plugin which skips
    all challenger, authentication and identifiers and returns
    the user contained in REMOTE_USER environment key.
    Has been made internal part of TG to make
    possible to switch to repoze.who v2
    """
    _HTTP_STATUS_PATTERN = re.compile(r'^(?P<code>[0-9]{3}) (?P<reason>.*)$')

    def __init__(self, fake_user_key='REMOTE_USER',
                 remote_user_key='repoze.who.testutil.userid'):
        self.fake_user_key = fake_user_key
        self.remote_user_key = remote_user_key

    def identify(self, environ):
        if self.fake_user_key in environ:
            identity = {'fake-userid': environ[self.fake_user_key]}
            return identity

    def remember(self, environ, identity): #pragma: no cover
        pass

    def forget(self, environ, identity): #pragma: no cover
        pass

    def authenticate(self, environ, identity):
        if 'fake-userid' in identity:
            environ[self.remote_user_key] = identity.pop('fake-userid')
            return environ[self.remote_user_key]

    def challenge(self, environ, status, app_headers, forget_headers):
        """Return a 401 page unconditionally."""
        headers = app_headers + forget_headers
        #remove content-length header
        headers = filter(lambda h:h[0].lower() != 'content-length', headers)

        # The HTTP status code and reason may not be the default ones:
        status_parts = self._HTTP_STATUS_PATTERN.search(status)
        reason = status_parts.group('reason')
        code = int(status_parts.group('code'))

        response = HTTPUnauthorized(headers=headers)
        response.title = reason
        response.code = code
        return response


class _AuthenticationForgerMiddleware(PluggableAuthenticationMiddleware):
    def __init__(self, app, identifiers, authenticators, challengers,
                 mdproviders, classifier, challenge_decider, log_stream=None,
                 log_level=logging.INFO, remote_user_key='REMOTE_USER'):
        """
        Took from repoze.who_testutil.
        This is meant for internal use only to setup tests with fake
        authentication. Has been made internal part of TG to make
        possible to switch to repoze.who v2
        """

        self.actual_remote_user_key = remote_user_key
        forger = _AuthenticationForgerPlugin(fake_user_key=remote_user_key)
        forger = ('auth_forger', forger)
        identifiers.insert(0, forger)
        authenticators = [forger]
        challengers = [forger]

        # Calling the parent's constructor:
        init = super(_AuthenticationForgerMiddleware, self).__init__
        init(app, identifiers, authenticators, challengers, mdproviders,
            classifier, challenge_decider, log_stream, log_level,
            'repoze.who.testutil.userid')


def turbogears_challenge_decider(environ, status, headers):
    if 'tg.skip_auth_challenge' in environ:
        return None

    return default_challenge_decider(environ, status, headers)


def setup_auth(app, form_plugin=None, form_identifies=True,
               cookie_secret='secret', cookie_name='authtkt',
               login_url='/login', login_handler='/login_handler',
               post_login_url=None, logout_handler='/logout_handler',
               post_logout_url=None, login_counter_name=None,
               cookie_timeout=None, cookie_reissue_time=None,
               **who_args):
    """
    Sets :mod:`repoze.who` up with the provided authenticators and
    options to create FriendlyFormPlugin/FastFormPlugin.

    It returns a middleware that provides identification,
    authentication and authorization in a way that is compatible
    with repoze.who and repoze.what.
    """
    if 'charset' in who_args: #pragma: no cover
        log.warn('charset argument in authentication setup is ignored')
        who_args.pop('charset')

    # If no identifiers are provided in repoze setup arguments
    # then create a default one using AuthTktCookiePlugin.
    identifiers = who_args.setdefault('identifiers', [('default', None)])
    try:
        default_identifier_index = identifiers.index(('default', None))
    except ValueError:
        pass
    else:
        # default identifier included.
        from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
        cookie = AuthTktCookiePlugin(cookie_secret, cookie_name,
                                     timeout=cookie_timeout,
                                     reissue_time=cookie_reissue_time)
        who_args['identifiers'][default_identifier_index] = ('cookie', cookie)
        who_args['authenticators'].insert(0, ('cookie', cookie))

    # If no form plugin is provided then create a default
    # one using the provided options.
    if form_plugin is None:
        from tg.configuration.auth.fastform import FastFormPlugin
        form = FastFormPlugin(login_url, login_handler, post_login_url,
                              logout_handler, post_logout_url,
                              rememberer_name='cookie',
                              login_counter_name=login_counter_name)
    else:
        form = form_plugin

    if form_identifies:
        who_args['identifiers'].insert(0, ('main_identifier', form))

    # Setting the repoze.who challengers:
    if 'challengers' not in who_args:
        who_args['challengers'] = []
    who_args['challengers'].append(('form', form))

    # Including logging
    log_file = who_args.pop('log_file', None)
    if log_file is not None:
        if log_file.lower() == 'stdout':
            log_stream = sys.stdout
        elif log_file.lower() == 'stderr':
            log_stream = sys.stderr
        else:
            log_stream = open(log_file, 'wb')
        who_args['log_stream'] = log_stream

    log_level = who_args.get('log_level', None)
    if log_level is None:
        log_level = logging.INFO
    else:
        log_level = _LEVELS[log_level.lower()]
    who_args['log_level'] = log_level

    # Setting up the metadata provider for the user information
    if 'mdproviders' not in who_args:
        who_args['mdproviders'] = []

    # Set up default classifier
    if 'classifier' not in who_args:
        who_args['classifier'] = default_request_classifier

    # Set up default challenger decider
    if 'challenge_decider' not in who_args:
        who_args['challenge_decider'] = turbogears_challenge_decider

    skip_authn = who_args.pop('skip_authentication', False)
    if asbool(skip_authn):
        return _AuthenticationForgerMiddleware(app, **who_args)
    else:
        return PluggableAuthenticationMiddleware(app, **who_args)



from nose.tools import raises
#from tg.configuration.utils import ConfigurationBunch


class TestConfigurationDictionary(object):
    def setup(self):
        self.d = ConfigurationBunch()

"""
    def test_plain_to_nested(self):
        d = self.d
        d['1st.2nd.3rd'] = 'HELLO'
        assert d['1st']['2nd']['3rd'] == 'HELLO'

    def test_nested_to_plain(self):
        d = self.d
        d['1st'] = {}
        d['1st']['2nd'] = {}
        d['1st']['2nd']['3rd'] = 'HELLO'
        assert d['1st.2nd.3rd'] == 'HELLO'

    def test_plain_to_plain(self):
        d = self.d
        d['1st.2nd.3rd'] = 'HELLO'
        assert d['1st.2nd.3rd'] == 'HELLO'

    def test_nested_to_attrs(self):
        d = self.d
        d['st1'] = {}
        d['st1']['nd2'] = {}
        d['st1']['nd2']['rd3'] = 'HELLO'
        assert d.st1.nd2.rd3 == 'HELLO'

    def test_plain_to_attrs(self):
        d = self.d
        d['st1.nd2.rd3'] = 'HELLO'
        assert d.st1.nd2.rd3 == 'HELLO'

    def test_iterate(self):
        d = self.d
        d['simple.sub'] = {'sub1': [3, 4], 'sub2': 'hi', 'sub3': {'subsub1': 7, 'subsub2': 8}}
        d['simple.sub.x'] = 'hi'

        flatkeys = list(iter(d))
        assert flatkeys == ['simple.sub.x', 'simple.sub.sub2', 'simple.sub.sub3.subsub2',
                            'simple.sub.sub3.subsub1', 'simple.sub.sub1'], flatkeys

    @raises(KeyError)
    def test_delete_plain(self):
        d = self.d
        d['st1'] = {}
        d['st1']['nd2'] = {}
        d['st1']['nd2']['rd3'] = 'HELLO'
        d.pop('st1.nd2.rd3', None)

        d['st1']['nd2']['rd3']

    @raises(AttributeError)
    def test_plain_to_attrs_not_found(self):
        d = self.d
        d['st1.nd2.rd3'] = 'HELLO'
        d.st1.nd2.rd4

    def test_set_plain(self):
        self.d['hi'] = 5
        self.d['simple.sub.sub1'] = [3, 4]
        self.d['simple.sub.sub2'] = [3, 4]
        assert sorted(list(self.d.keys())) == sorted(['hi', 'simple']), self.d
        assert 'sub' in self.d['simple'], self.d
        assert len(self.d['simple.sub'].keys()) == 2

    def test_set_subdict(self):
        d = self.d
        d['simple.sub'] = {'sub1': [3, 4], 'sub2': 'hi', 'sub3': {'subsub1': 7, 'subsub2': 8}}
        d['simple.sub.x'] = 'hi'
        assert len(d) == 1, d
        assert d['simple'] == {'sub': {'x': 'hi', 'sub1': [3, 4], 'sub2': 'hi', 'sub3': {'subsub1': 7, 'subsub2': 8}}}

    def test_get_subdict(self):
        self.test_set_subdict()
        assert self.d['simple.sub.sub3'] == {'subsub1': 7, 'subsub2': 8}

    @raises(TypeError)
    def test_odd_parents_and_children1(self):
        self.d['parent'] = 5
        self.d['parent.child'] = 3

    @raises(TypeError)
    def test_odd_parents_and_children2(self):
        self.d['parent.child'] = 3
        self.d['parent'] = 3

        # parent is now an int so it should fail
        self.d['parent.child']
"""


from __future__ import absolute_import

from markupsafe import Markup
from tg.configuration.utils import coerce_config
from tg.support.converters import asint, asbool
from tg.i18n import ugettext
from tg.render import cached_template
from .base import RendererFactory
import tg

try:
    import genshi
except ImportError:  # pragma: no cover
    genshi = None

if genshi is not None:
    from genshi.template import TemplateLoader as GenshiTemplateLoader
    from genshi.filters import Translator
    from genshi import HTML, XML
else:  # pragma: no cover
    class GenshiTemplateLoader(object): pass


__all__ = ['GenshiRenderer']


class GenshiRenderer(RendererFactory):
    """
    Configuration Options available as ``templating.genshi.*``:

        - ``templating.genshi.name_constant_patch`` -> Enable/Disable patch for Python3.4 compatibility.
        - ``templating.genshi.max_cache_size`` -> Maximum number of templates to keep cached, by default 30.
        - ``templating.genshi.method`` -> Genshi rendering method (html or xhtml).

    Supported ``render_params``:

        - Caching options supported by :func:`.cached_template`
        - ``doctype`` -> To override the global doctype
        - ``method`` -> To override the global rendering method
    """
    CONFIG_OPTIONS = {
        'max_cache_size': asint,
        'name_constant_patch': asbool
    }
    engines = {'genshi': {'content_type': 'text/html'}}

    doctypes_for_methods = {
        'html': 'html-transitional',
        'xhtml': 'xhtml-transitional'}

    doctypes_for_content_type = {
        'text/html': ('html', 'html-transitional',
            'html-frameset', 'html5',
            'xhtml', 'xhtml-strict',
            'xhtml-transitional', 'xhtml-frameset'),
        'application/xhtml+xml': ('xhtml', 'xhtml-strict',
            'xhtml-transitional',
            'xhtml-frameset', 'xhtml11'),
        'image/svg+xml': ('svg', 'svg-full', 'svg-basic', 'svg-tiny')}

    methods_for_content_type = {
        'text/plain': ('text',),
        'text/css': ('text',),
        'text/html': ('html', 'xhtml'),
        'text/xml': ('xml', 'xhtml'),
        'application/xml': ('xml', 'xhtml'),
        'application/xhtml+xml': ('xhtml',),
        'application/atom+xml': ('xml',),
        'application/rss+xml': ('xml',),
        'application/soap+xml': ('xml',),
        'image/svg+xml': ('xml',)}

    @classmethod
    def create(cls, config, app_globals):
        """Setup a renderer and loader for Genshi templates.

        Override this to customize the way that the internationalization
        filter, template loader

        """
        if genshi is None:  # pragma: no cover
            # Genshi not available
            return None

        options = coerce_config(config, 'templating.genshi.', cls.CONFIG_OPTIONS)

        # Patch for Genshi on Python3.4
        if options.get('name_constant_patch', False):
            from genshi.template.astutil import ASTCodeGenerator
            if not hasattr(ASTCodeGenerator, 'visit_NameConstant'):
                def _visit_NameConstant(self, node):
                    if node.value is None:
                        self._write('None')
                    elif node.value is True:
                        self._write('True')
                    elif node.value is False:
                        self._write('False')
                    else:
                        raise Exception("Unknown NameConstant %r" % (node.value,))
                ASTCodeGenerator.visit_NameConstant = _visit_NameConstant

        if config.get('use_dotted_templatenames', True):
            TemplateLoader = DottedTemplateLoader
            template_loader_args = {'dotted_finder': app_globals.dotted_filename_finder}
        else:
            TemplateLoader = GenshiTemplateLoader
            template_loader_args = {}

        loader = TemplateLoader(search_path=config['paths'].templates,
                                max_cache_size=options.get(
                                    'max_cache_size',
                                    asint(config.get('genshi.max_cache_size', 30))
                                ),
                                auto_reload=config['auto_reload_templates'],
                                callback=cls.on_template_loaded,
                                **template_loader_args)

        return {'genshi': cls(loader, config)}

    def __init__(self, loader, config):
        self.tg_config = config
        self.genshi_functions = dict(HTML=HTML, XML=XML)
        self.load_template = loader.load

        self.default_doctype = None
        doctype = self.tg_config.get('templating.genshi.doctype')
        if doctype:
            if isinstance(doctype, str):
                self.default_doctype = doctype
            elif isinstance(doctype, dict):
                doctypes = self.doctypes_for_content_type.copy()
                doctypes.update(doctype)
                self.doctypes_for_content_type = doctypes

        self.default_method = None
        method = self.tg_config.get('templating.genshi.method')
        if method:
            if isinstance(method, str):
                self.default_method = method
            elif isinstance(method, dict):
                methods = self.methods_for_content_type.copy()
                methods.update(method)
                self.methods_for_content_type = methods

    @classmethod
    def on_template_loaded(cls, template):
        """
        Plug-in our i18n function to Genshi, once the template is loaded.

        This function will be called by the Genshi TemplateLoader after
        loading the template.

        """
        translator = Translator(ugettext)
        template.filters.insert(0, translator)

        if hasattr(template, 'add_directives'):
            template.add_directives(Translator.NAMESPACE, translator)

    @staticmethod
    def method_for_doctype(doctype):
        method = 'xhtml'
        if doctype:
            if doctype.startswith('html'):
                method = 'html'
            elif doctype.startswith('xhtml'):
                method = 'xhtml'
            elif doctype.startswith('svg'):
                method = 'xml'
            else:
                method = 'xhtml'
        return method

    def __call__(self, template_name, template_vars, **kwargs):
        """Render the template_vars with the Genshi template.

        If you don't pass a doctype or pass 'auto' as the doctype,
        then the doctype will be automatically determined.
        If you pass a doctype of None, then no doctype will be injected.
        If you don't pass a method or pass 'auto' as the method,
        then the method will be automatically determined.

        """
        response = tg.response._current_obj()

        template_vars.update(self.genshi_functions)

        # Gets document type from content type or from config options
        doctype = kwargs.get('doctype', 'auto')
        if doctype == 'auto':
            doctype = self.default_doctype
            if not doctype:
                method = kwargs.get('method') or self.default_method or 'xhtml'
                doctype = self.doctypes_for_methods.get(method)
            doctypes = self.doctypes_for_content_type.get(response.content_type)
            if doctypes and (not doctype or doctype not in doctypes):
                doctype = doctypes[0]
            kwargs['doctype'] = doctype

        # Gets rendering method from content type or from config options
        method = kwargs.get('method')
        if not method or method == 'auto':
            method = self.default_method
            if not method:
                method = self.method_for_doctype(doctype)
            methods = self.methods_for_content_type.get(response.content_type)
            if methods and (not method or method not in methods):
                method = methods[0]
            kwargs['method'] = method

        def render_template():
            template = self.load_template(template_name)
            return Markup(template.generate(**template_vars).render(
                doctype=doctype,
                method=method,
                encoding=None)
            )

        return cached_template(template_name, render_template,
                               ns_options=('doctype', 'method'), **kwargs)


class DottedTemplateLoader(GenshiTemplateLoader):
    """
    Genshi template loader supporting dotted filenames.
    Supports zipped applications and dotted filenames as well as path names.

    """
    def __init__(self, *args, **kwargs):
        self.template_extension = kwargs.pop('template_extension', '.html')
        self.dotted_finder = kwargs.pop('dotted_finder')

        super(DottedTemplateLoader, self).__init__(*args, **kwargs)

    def get_dotted_filename(self, filename):
        if not filename.endswith(self.template_extension):
            finder = self.dotted_finder
            filename = finder.get_dotted_filename(template_name=filename,
                                                  template_extension=self.template_extension)
        return filename

    def load(self, filename, relative_to=None, cls=None, encoding=None):
        """Actual loader function."""
        return super(DottedTemplateLoader, self).load(self.get_dotted_filename(filename),
                                                      relative_to=relative_to, cls=cls,
                                                      encoding=encoding)



# -*- coding: utf-8 -*-
from nose.tools import raises
import os, tg
from tests.test_stack import TestConfig, app_from_config
from tg.configuration.hooks import _TGGlobalHooksNamespace
from tg.decorators import Decoration
from tg.configuration import milestones
from tg.util import Bunch


class TestHooks(object):
    def setUp(self):
        milestones._reset_all()
        tg.hooks = _TGGlobalHooksNamespace()

    def tearDown(self):
        milestones._reset_all()
        tg.hooks = _TGGlobalHooksNamespace()

    def test_hooks_syswide(self):
        base_config = TestConfig(folder = 'dispatch',
                                 values = {'use_sqlalchemy': False,
                                           'use_toscawidgets': False,
                                           'use_toscawidgets2': False,
                                           'ignore_parameters': ["ignore", "ignore_me"]
                                 })

        def hook(*args, **kw):
            tg.tmpl_context.echo = 'WORKED'

        base_config.register_hook('before_call', hook)
        app = app_from_config(base_config, reset_milestones=False)

        ans = app.get('/echo')
        assert 'WORKED' in ans

    def test_decoration_run_hooks_backward_compatibility(self):
        # TODO: Remove test when Decoration.run_hooks gets removed

        def func(*args, **kw):
            pass

        def hook(*args, **kw):
            hook.did_run = True
        hook.did_run = False

        milestones.renderers_ready.reach()
        tg.hooks.register('before_call', hook, controller=func)

        deco = Decoration.get_decoration(func)
        deco.run_hooks(Bunch(config=None), 'before_call')

        assert hook.did_run is True

class TestExpose(object):
    def setUp(self):
        milestones.renderers_ready._reset()

    def tearDown(self):
        milestones.renderers_ready._reset()

    def test_unregisterd_renderers_detection(self):
        #If no renderers are available we should just issue a warning
        #and avoid crashing. Simply bypass rendering availability check.
        base_config = TestConfig(folder = 'dispatch',
            values = {'use_sqlalchemy': False,
                      'use_toscawidgets': False,
                      'use_toscawidgets2': False,
                      'ignore_parameters': ["ignore", "ignore_me"]
            })

        app = app_from_config(base_config)

        old_renderers = tg.config['renderers']
        tg.config['renderers'] = []

        @tg.expose('mako:nonexisting')
        def func(*args, **kw):
            pass

        tg.config['renderers'] = old_renderers

    def test_use_default_renderer(self):
        base_config = TestConfig(folder = 'dispatch',
            values = {'use_sqlalchemy': False,
                      'use_toscawidgets': False,
                      'use_toscawidgets2': False,
                      'ignore_parameters': ["ignore", "ignore_me"]
            })

        app = app_from_config(base_config)

        exposition = tg.expose('nonexisting')
        exposition._resolve_options()

        assert exposition.engine == tg.config['default_renderer']
        assert exposition.template == 'nonexisting'

    def test_expose_without_function_does_nothing(self):
        base_config = TestConfig(folder = 'dispatch',
            values = {'use_sqlalchemy': False,
                      'use_toscawidgets': False,
                      'use_toscawidgets2': False,
                      'ignore_parameters': ["ignore", "ignore_me"]
            })

        app = app_from_config(base_config)

        exposition = tg.expose('nonexisting')
        exposition._apply()

        assert exposition._func is None
        assert exposition.engine is None

    def test_expose_idempotent(self):
        base_config = TestConfig(folder = 'dispatch',
            values = {'use_sqlalchemy': False,
                      'use_toscawidgets': False,
                      'use_toscawidgets2': False,
                      'ignore_parameters': ["ignore", "ignore_me"]
            })

        app = app_from_config(base_config)

        exposition = tg.expose('nonexisting')

        @exposition
        @exposition
        def func(*args, **kw):
            pass

        milestones.renderers_ready.reach()

        deco = Decoration.get_decoration(func)
        assert len(deco.engines) == 1, deco.engines

class TestDecorators(object):
    def setup(self):
        base_config = TestConfig(folder = 'dispatch',
            values = {'use_sqlalchemy': False,
                      'use_toscawidgets': False,
                      'use_toscawidgets2': False,
                      'ignore_parameters': ["ignore", "ignore_me"]
            })

        self.app = app_from_config(base_config)

    def test_variabledecode_fail(self):
        resp = self.app.get('/test_vardec', params={'test-1': '1',
                                                    'test-2': 2,
                                                    'test--repetitions': 'hi'})
        assert resp.json['test-1'] == '1', resp.json
        assert resp.json['test--repetitions'] == 'hi', resp.json
        assert 'test' not in resp.json, resp.json

    def test_variabledecode_partial_fail(self):
        resp = self.app.get('/test_vardec', params={'test-1': '1',
                                                    'test-2': 2,
                                                    'test-': 4})
        assert resp.json['test-1'] == '1'
        assert resp.json['test-'] == '4'
        assert len(resp.json['test']) == 2

    def test_variable_decode(self):
        from formencode.variabledecode import variable_encode
        obj = dict(a=['1','2','3'], b=dict(c=[dict(d='1')]))
        params = variable_encode(dict(obj=obj), add_repetitions=False)
        resp = self.app.get('/test_vardec', params=params)
        assert resp.json['obj'] == obj, (resp.json['obj'], obj)

    def test_without_trailing_slash(self):
        resp = self.app.get('/without_tslash/', status=301)
        assert resp.headers['Location'].endswith('/without_tslash')

    def test_with_trailing_slash(self):
        resp = self.app.get('/with_tslash', status=301)
        assert resp.headers['Location'].endswith('/with_tslash/')

    def test_with_engine(self):
        resp = self.app.get('/onmaster')
        assert 'mainslave' in resp

    def test_with_engine_without_params(self):
        resp = self.app.get('/onmaster_without_params?first=1&second=1')
        assert 'mainslave' in resp

    def test_with_engine_nopop(self):
        resp = self.app.get('/onmaster?second=1')
        assert 'master' in resp
        assert 'second' in resp

    def test_with_engine_pop(self):
        resp = self.app.get('/onmaster?first=1')
        assert 'master' in resp
        assert 'first' not in resp

    def test_with_engine_using_list(self):
        resp = self.app.get('/onmaster_withlist?first=1')
        assert 'master' in resp
        assert 'first' not in resp



# -*- coding: utf-8 -*-
from tgming.auth import MingAuthenticatorPlugin

def create_default_authenticator(user_class, translations=None, **unused):
    mingauth = MingAuthenticatorPlugin(user_class)
    return unused, mingauth



import os
from tests.test_stack import TestConfig
from webtest import TestApp

def setup_noDB():
    global_config = {'debug': 'true', 
                     'error_email_from': 'paste@localhost', 
                     'smtp_server': 'localhost'}
    
    base_config = TestConfig(folder = 'config', 
                             values = {'use_sqlalchemy': False,
                                       'use_toscawidgets':False,
                                       'use_toscawidgets2':False}
                             )
                             
    env_loader = base_config.make_load_environment()
    app_maker = base_config.setup_tg_wsgi_app(env_loader)
    app = TestApp(app_maker(global_config, full_stack=True))
    return app 

def test_basic_stack():
    app = setup_noDB()
    resp = app.get('/')
    assert resp.body.decode('ascii') == "my foo"

def test_config_reading():
    """Ensure that the config object can be read via dict and attr access"""
    app = setup_noDB()
    resp = app.get('/config_test')
    assert "default_renderer" in str(resp.body)
    resp = app.get('/config_attr_lookup')
    assert "genshi" in str(resp.body)
    resp = app.get('/config_dotted_values')
    assert "root" in str(resp.body)

def test_config_writing():
    """Ensure that new values can be added to the config object"""
    app = setup_noDB()
    value = "gooberblue"
    resp = app.get('/config_attr_set/'+value)
    assert value in str(resp.body)
    resp = app.get('/config_dict_set/'+value)
    assert value in str(resp.body)




from tg.controllers.util import _build_url

try:
    from urlparse import urlparse, urlunparse, parse_qs
except ImportError: #pragma: no cover
    from urllib.parse import urlparse, urlunparse, parse_qs

try:
    from urllib import urlencode
except ImportError: #pragma: no cover
    from urllib.parse import urlencode

from webob import Request
from webob.exc import HTTPFound, HTTPUnauthorized
from zope.interface import implementer

from repoze.who.interfaces import IChallenger, IIdentifier

@implementer(IChallenger, IIdentifier)
class FastFormPlugin(object):
    """
    Simplified and faster version of the repoze.who.friendlyforms
    FriendlyForm plugin. The FastForm version works only with UTF-8
    content which is the default for new WebOb versions.
    """
    classifications = {
        IIdentifier: ["browser"],
        IChallenger: ["browser"],
        }

    def __init__(self, login_form_url, login_handler_path, post_login_url,
                 logout_handler_path, post_logout_url, rememberer_name,
                 login_counter_name=None):
        """
        :param login_form_url: The URL/path where the login form is located.
        :type login_form_url: str
        :param login_handler_path: The URL/path where the login form is
            submitted to (where it is processed by this plugin).
        :type login_handler_path: str
        :param post_login_url: The URL/path where the user should be redirected
            to after login (even if wrong credentials were provided).
        :type post_login_url: str
        :param logout_handler_path: The URL/path where the user is logged out.
        :type logout_handler_path: str
        :param post_logout_url: The URL/path where the user should be
            redirected to after logout.
        :type post_logout_url: str
        :param rememberer_name: The name of the repoze.who identifier which
            acts as rememberer.
        :type rememberer_name: str
        """
        self.login_form_url = login_form_url
        self.login_handler_path = login_handler_path
        self.post_login_url = post_login_url
        self.logout_handler_path = logout_handler_path
        self.post_logout_url = post_logout_url
        self.rememberer_name = rememberer_name

        if not login_counter_name:
            login_counter_name = '__logins'
        self.login_counter_name = login_counter_name

    # IIdentifier
    def identify(self, environ):
        path_info = environ['PATH_INFO']

        if path_info == self.login_handler_path:
            query = self._get_form_data(environ)

            try:
                credentials = {'login': query['login'],
                               'password': query['password'],
                               'max_age':query.get('remember')}
            except KeyError:
                credentials = None

            params = {}
            if 'came_from' in query:
                params['came_from'] = query['came_from']
            if self.login_counter_name is not None and self.login_counter_name in query:
                params[self.login_counter_name] = query[self.login_counter_name]

            destination = _build_url(environ, self.post_login_url, params=params)
            environ['repoze.who.application'] = HTTPFound(location=destination)
            return credentials

        elif path_info == self.logout_handler_path:
            query = self._get_form_data(environ)
            came_from = query.get('came_from')
            if came_from is None:
                came_from = _build_url(environ, '/')

            # set in environ for self.challenge() to find later
            environ['came_from'] = came_from
            environ['repoze.who.application'] = HTTPUnauthorized()

        elif path_info in (self.login_form_url, self.post_login_url):
            query = self._get_form_data(environ)
            environ['repoze.who.logins'] = 0

            if self.login_counter_name is not None and self.login_counter_name in query:
                environ['repoze.who.logins'] = int(query[self.login_counter_name])
                del query[self.login_counter_name]
                environ['QUERY_STRING'] = urlencode(query, doseq=True)

        return None

    # IChallenger
    def challenge(self, environ, status, app_headers, forget_headers):
        path_info = environ['PATH_INFO']

        # Configuring the headers to be set:
        cookies = [(h,v) for (h,v) in app_headers if h.lower() == 'set-cookie']
        headers = forget_headers + cookies

        if path_info == self.logout_handler_path:
            params = {}
            if 'came_from' in environ:
                params.update({'came_from':environ['came_from']})
            destination = _build_url(environ, self.post_logout_url, params=params)

        else:
            came_from_params = parse_qs(environ.get('QUERY_STRING', ''))
            params = {'came_from': _build_url(environ, path_info, came_from_params)}
            destination = _build_url(environ, self.login_form_url, params=params)

        return HTTPFound(location=destination, headers=headers)

    # IIdentifier
    def remember(self, environ, identity):
        rememberer = self._get_rememberer(environ)
        return rememberer.remember(environ, identity)

    # IIdentifier
    def forget(self, environ, identity):
        rememberer = self._get_rememberer(environ)
        return rememberer.forget(environ, identity)

    def _get_rememberer(self, environ):
        rememberer = environ['repoze.who.plugins'][self.rememberer_name]
        return rememberer

    def _get_form_data(self, environ):
        request = Request(environ)
        query = dict(request.GET)
        query.update(request.POST)
        return query

    def __repr__(self):
        return '<%s:%s %s>' % (self.__class__.__name__, self.login_handler_path, id(self))



# -*- coding: utf-8 -*-

from webob import Response, Request

from tg.controllers import TGController, RestController
from tg.decorators import expose
from tg.util import no_warn

from tests.base import (
    TestWSGIController, make_app, setup_session_dir, teardown_session_dir)


def setup():
    setup_session_dir()

def teardown():
    teardown_session_dir()


def wsgi_app(environ, start_response):
    req = Request(environ)
    if req.method == 'POST':
        resp = Response(req.POST['data'])
    else:
        resp = Response("Hello from %s/%s"%(req.script_name, req.path_info))
    return resp(environ, start_response)


class LookupHelper:
    def __init__(self, var):
        self.var = var

    @expose()
    def index(self):
        return self.var


class LookupController(TGController):

    @expose()
    def _lookup(self, a, *args):
        return LookupHelper(a), args


class DeprecatedLookupController(TGController):

    @expose()
    def _lookup(self, a, *args):
        return LookupHelper(a), args


class LookupAlwaysHelper:
    """for testing _dispatch"""

    def __init__(self, var):
        self.var = var

    def _setup_wsgiorg_routing_args(self, url_path, remainder, params):
        pass

    @expose()
    def always(self, *args, **kwargs):
        return 'always go here'

    def _dispatch(self, state, remainder):
        state.add_method(self.always, remainder)
        return state


class LookupAlwaysController(TGController):

    @expose()
    def _lookup(self, a, *args):
        return LookupAlwaysHelper(a), args


class CustomDispatchingSubController(TGController):

    @expose()
    def always(self, *args, **kwargs):
        return 'always go here'

    def _dispatch(self, state, remainder):
        state.add_method(self.always, remainder)
        return state


class OptionalArgumentRestController(RestController):

    @expose()
    def get_one(self, optional=None):
        return "SUBREST GET ONE"

    @expose()
    def put(self, optional=None):
        return "subrest put"

    @expose()
    def post(self, optional=None):
        return "subrest post"

    @expose()
    def edit(self, optional=None):
        return "subrest edit"

    @expose()
    def new(self, optional=None):
        return "subrest new"

    @expose()
    def get_delete(self, optional=None):
        return "subrest get delete"

    @expose()
    def post_delete(self, optional=None):
        return "subrest post delete"


class RequiredArgumentRestController(RestController):

    @expose()
    def get_one(self, something):
        return "subrest get one"

    @expose()
    def put(self, something):
        return "subrest put"

    @expose()
    def post(self, something):
        return "subrest post"

    @expose()
    def edit(self, something):
        return "subrest edit"

    @expose()
    def new(self):
        return "subrest new"

    @expose()
    def get_delete(self, something):
        return "subrest get delete"

    @expose()
    def post_delete(self, something):
        return "subrest post delete"


class VariableSubRestController(RestController):

    @expose()
    def get_one(self, *args):
        return "subrest get one"

    @expose()
    def put(self, *args):
        return "subrest put"

    @expose()
    def edit(self, *args):
        return "subrest edit"

    @expose()
    def new(self, *args):
        return "subrest new"

    @expose()
    def get_delete(self, *args):
        return "subrest get delete"

    @expose()
    def post_delete(self, *args):
        return "subrest post delete"


class SubRestController(RestController):

    @expose()
    def get_all(self):
        return "subrest get all"

    @expose()
    def get_one(self, nr):
        return "subrest get one %s" % nr

    @expose()
    def new(self):
        return "subrest new"

    @expose()
    def edit(self, nr):
        return "subrest edit %s" % nr

    @expose()
    def post(self):
        return "subrest post"

    @expose()
    def put(self, nr):
        return "subrest put %s" % nr

    @expose()
    def fxn(self):
        return "subrest fxn"

    @expose()
    def get_delete(self, nr):
        return "subrest get delete %s" % nr

    @expose()
    def post_delete(self, nr):
        return "subrest post delete %s" % nr


class VariableRestController(RestController):

    subrest = SubRestController()
    vsubrest = VariableSubRestController()

    @expose()
    def get_all(self):
        return "rest get all"

    @expose()
    def get_one(self, *args):
        return "rest get onE"

    @expose()
    def get_delete(self, *args):
        return "rest get delete"

    @expose()
    def post_delete(self, *args):
        return "rest post delete"


class ExtraRestController(RestController):

    @expose()
    def get_all(self):
        return "rest get all"

    @expose()
    def get_one(self, nr):
        return "rest get one %s" % nr

    @expose()
    def get_delete(self, nr):
        return "rest get delete %s" % nr

    @expose()
    def post_delete(self, nr):
        return "rest post delete %s" % nr

    class SubClass(TGController):
        @expose()
        def index(self):
            return "rest sub index"

    sub = SubClass()
    subrest = SubRestController()
    optsubrest = OptionalArgumentRestController()
    reqsubrest = RequiredArgumentRestController()

    @expose()
    def post_archive(self):
        return 'got to post archive'

    @expose()
    def get_archive(self):
        return 'got to get archive'


class BasicRestController(RestController):

    @expose()
    def get(self):
        return "rest get"

    @expose()
    def post(self):
        return "rest post"

    @expose()
    def put(self):
        return "rest put"

    @expose()
    def delete(self):
        return "rest delete"

    @expose()
    def new(self):
        return "rest new"
    @expose()
    def edit(self, *args, **kw):
        return "rest edit"

    @expose()
    def other(self):
        return "rest other"

    @expose()
    def archive(self):
        return 'got to archive'


class EmptyRestController(RestController):
    pass


class SubController(TGController):

    rest = BasicRestController()

    @expose()
    def sub_method(self, arg):
        return 'sub %s'%arg


class BasicTGController(TGController):

    sub = SubController()
    custom_dispatch = CustomDispatchingSubController()
    lookup = LookupController()
    deprecated_lookup = LookupController()
    lookup_dispatch = LookupAlwaysController()
    rest  = BasicRestController()
    rest2 = ExtraRestController()
    rest3 = VariableRestController()
    empty = EmptyRestController()

    @expose()
    def index(self, **kwargs):
        return 'hello world'

    @expose()
    def _default(self, *remainder):
        return "Main default page called for url /%s" % [str(r) for r in remainder]

    @expose()
    def hello(self, name, silly=None):
        return "Hello %s" % name


class BasicTGControllerNoDefault(TGController):

    @expose()
    def index(self, **kwargs):
        return 'hello world'


class TestTGControllerRoot(TestWSGIController):

    def __init__(self, *args, **kargs):
        TestWSGIController.__init__(self, *args, **kargs)
        self.app = make_app(BasicTGControllerNoDefault)

    def test_root_default_dispatch(self):
        self.app.get('/i/am/not/a/sub/controller', status=404)


class TestTGController(TestWSGIController):

    def __init__(self, *args, **kargs):
        TestWSGIController.__init__(self, *args, **kargs)
        self.app = make_app(BasicTGController)

    def test_lookup(self):
        r = self.app.get('/lookup/eye')
        msg = 'eye'
        assert msg in r, r

    def test_deprecated_lookup(self):
        r = self.app.get('/deprecated_lookup/eye')
        msg = 'eye'
        assert msg in r, r

    def test_lookup_with_dispatch(self):
        r = self.app.get('/lookup_dispatch/eye')
        msg = 'always'
        assert msg in r, r

    def test_root_method_dispatch(self):
        resp = self.app.get('/hello/Bob')
        assert "Hello Bob" in resp, resp

    def test_root_index_dispatch(self):
        resp = self.app.get('/')
        assert "hello world" in resp, resp

    def test_no_sub_index_dispatch(self):
        resp = self.app.get('/sub/')
        assert "['sub']" in resp, resp

    def test_root_default_dispatch(self):
        resp = self.app.get('/i/am/not/a/sub/controller')
        assert "['i', 'am', 'not', 'a', 'sub', 'controller']" in resp, resp

    def test_default_dispatch_not_found_in_sub_controller(self):
        resp = self.app.get('/sub/no/default/found')
        assert "['sub', 'no', 'default', 'found']" in resp, resp

    def test_root_method_dispatch_with_trailing_slash(self):
        resp = self.app.get('/hello/Bob/')
        assert "Hello Bob" in resp, resp

    def test_sub_method_dispatch(self):
        resp = self.app.get('/sub/sub_method/army of darkness')
        assert "sub army" in resp, resp

    def test_custom_dispatch(self):
        resp = self.app.get('/custom_dispatch/army of darkness')
        assert "always" in resp, resp

class TestRestController(TestWSGIController):

    def __init__(self, *args, **kargs):
        TestWSGIController.__init__(self, *args, **kargs)
        self.app = make_app(BasicTGController)

    def test_post(self):
        r = self.app.post('/rest/')
        assert 'rest post' in r, r

    def _test_non_resty(self):
        r = self.app.post('/rest/non_resty_thing')
        assert 'non_resty' in r, r

    def test_custom_action_simple_get(self):
        r = self.app.get('/rest/archive')
        assert 'got to archive' in r, r

    def test_custom_action_simple_post(self):
        r = self.app.post('/rest/archive')
        assert 'got to archive' in r, r

    def test_custom_action_simple_post_args(self):
        r = self.app.post('/rest?_method=archive')
        assert 'got to archive' in r, r

    def test_custom_action_get(self):
        r = self.app.get('/rest2/archive')
        assert 'got to get archive' in r, r

    def test_custom_action_post(self):
        r = self.app.post('/rest2?_method=archive')
        assert 'got to post archive' in r, r

    def test_get(self):
        r = self.app.get('/rest/')
        assert 'rest get' in r, r

    def test_put(self):
        r = self.app.put('/rest/')
        assert 'rest put' in r, r

    def test_put_post(self):
        r = self.app.post('/rest?_method=PUT')
        assert 'rest put' in r, r

    def test_put_post_params(self):
        r = self.app.post('/rest', params={'_method':'PUT'})
        assert 'rest put' in r, r

    def test_put_get(self):
        self.app.get('/rest?_method=PUT', status=405)

    def test_get_delete_bad(self):
        self.app.get('/rest?_method=DELETE', status=405)

    def test_delete(self):
        r = self.app.delete('/rest/')
        assert 'rest delete' in r, r

    def test_post_delete(self):
        r = self.app.post('/rest/', params={'_method':'DELETE'})
        assert 'rest delete' in r, r

    def test_get_all(self):
        r = self.app.get('/rest2/')
        assert 'rest get all' in r, r

    def test_get_one(self):
        r = self.app.get('/rest2/1')
        assert 'rest get one 1' in r, r

    def test_get_delete(self):
        r = self.app.get('/rest2/1/delete')
        assert 'rest get delete' in r, r

    def test_post_delete_params(self):
        r = self.app.post('/rest2/1', params={'_method':'DELETE'})
        assert 'rest post delete' in r, r

    def test_post_delete_var(self):
        r = self.app.post('/rest3/a/b/c', params={'_method':'DELETE'})
        assert 'rest post delete' in r, r

    def test_get_delete_var(self):
        r = self.app.get('/rest3/a/b/c/delete')
        assert 'rest get delete' in r, r

    def test_get_method(self):
        r = self.app.get('/rest/other')
        assert 'rest other' in r, r

    @no_warn
    def test_get_sub_controller(self):
        r = self.app.get('/rest2/sub')
        assert 'rest sub index' in r, r

    @no_warn
    def test_put_sub_controller(self):
        r = self.app.put('/rest2/sub')
        assert 'rest sub index' in r, r

    def test_post_sub_controller(self):
        r = self.app.post('/rest2/sub')
        assert 'rest sub index' in r, r

    def test_post_miss(self):
        r = self.app.post('/rest2/something')
        assert "/['rest2', 'something']" in r, r

    def test_get_empty(self):
        r = self.app.get('/empty/')
        assert "/['empty']" in r, r

    def test_post_empty(self):
        r = self.app.post('/empty/')
        assert "/['empty']" in r, r

    def test_put_empty(self):
        r = self.app.put('/empty/')
        assert "/['empty']" in r, r

    @no_warn
    def test_delete_empty(self):
        r = self.app.delete('/empty/')
        assert "/['empty']" in r, r

    def test_put_miss(self):
        r = self.app.put('/rest/something')
        assert "/['rest', 'something']" in r, r

    def test_delete_miss(self):
        r = self.app.delete('/rest/something')
        assert "/['rest', 'something']" in r, r

    def test_get_miss(self):
        r = self.app.get('/rest2/something/else')
        assert "/['rest2', 'something', 'else']" in r, r

    def test_post_method(self):
        r = self.app.post('/rest/other')
        assert 'rest other' in r, r

    def test_new_method(self):
        r = self.app.post('/rest/new')
        assert 'rest new' in r, r

    def test_edit_method(self):
        r = self.app.get('/rest/1/edit')
        assert 'rest edit' in r, r

    def test_delete_method(self):
        self.app.delete('/rest/other', status=405)

    def test_sub_with_rest_delete(self):
        r = self.app.delete('/sub/rest/')
        assert 'rest delete' in r, r

    def test_put_method(self):
        r = self.app.put('/rest/other')
        assert 'rest other' in r, r

    def test_sub_get_all_method(self):
        r = self.app.get('/rest2/1/subrest')
        assert 'subrest get all' in r, r

    def test_var_sub_get_all_method(self):
        r = self.app.get('/rest3/1/3/3/subrest')
        assert 'subrest get all' in r, r
        r = self.app.get('/rest3/1/3/subrest')
        assert 'subrest get all' in r, r
        r = self.app.get('/rest3/subrest')
        assert 'subrest get all' in r, r

    def test_var_sub_get_one_method(self):
        r = self.app.get('/rest3/1/3/3/subrest/1')
        assert 'subrest get one' in r, r
        r = self.app.get('/rest3/1/3/subrest/1')
        assert 'subrest get one' in r, r
        r = self.app.get('/rest3/subrest/1')
        assert 'subrest get one' in r, r

    def test_var_sub_edit_method(self):
        r = self.app.get('/rest3/1/3/3/subrest/1/edit')
        assert 'subrest edit' in r, r
        r = self.app.get('/rest3/1/3/subrest/1/edit')
        assert 'subrest edit' in r, r
        r = self.app.get('/rest3/subrest/1/edit')
        assert 'subrest edit' in r, r

    def test_var_sub_edit_var_method(self):
        r = self.app.get('/rest3/1/3/3/vsubrest/1/edit')
        assert 'subrest edit' in r, r
        r = self.app.get('/rest3/1/3/vsubrest/1/a/edit')
        assert 'subrest edit' in r, r
        r = self.app.get('/rest3/vsubrest/edit')
        assert 'subrest edit' in r, r

    def test_var_sub_delete_method(self):
        r = self.app.get('/rest3/1/3/3/subrest/1/delete')
        assert 'subrest get delete' in r, r
        r = self.app.get('/rest3/1/3/subrest/1/delete')
        assert 'subrest get delete' in r, r
        r = self.app.get('/rest3/subrest/1/delete')
        assert 'subrest get delete' in r, r

    def test_var_sub_new_method(self):
        r = self.app.get('/rest3/1/3/3/subrest/new')
        assert 'subrest new' in r, r
        r = self.app.get('/rest3/1/3/subrest/new')
        assert 'subrest new' in r, r
        r = self.app.get('/rest3/subrest/new')
        assert 'subrest new' in r, r

    def test_var_sub_var_get_one_method(self):
        r = self.app.get('/rest3/1/3/3/vsubrest/1')
        assert 'subrest get one' in r, r
        r = self.app.get('/rest3/1/3/vsubrest/1/a')
        assert 'subrest get one' in r, r
        r = self.app.get('/rest3/vsubrest/')
        assert 'subrest get one' in r, r

    def test_var_sub_var_put_method(self):
        r = self.app.put('/rest3/1/3/3/vsubrest/1')
        assert 'subrest put' in r, r
        r = self.app.put('/rest3/1/3/vsubrest/1/asdf')
        assert 'subrest put' in r, r
        r = self.app.put('/rest3/vsubrest/')
        assert 'subrest put' in r, r

    def test_var_sub_post_method(self):
        r = self.app.post('/rest3/1/3/3/subrest/')
        assert 'subrest post' in r, r
        r = self.app.post('/rest3/1/3/subrest/')
        assert 'subrest post' in r, r
        r = self.app.post('/rest3/subrest/')
        assert 'subrest post' in r, r

    def test_var_sub_post_delete_method(self):
        r = self.app.delete('/rest3/1/3/3/subrest/1')
        assert 'subrest post delete' in r, r
        r = self.app.delete('/rest3/1/3/subrest/1')
        assert 'subrest post delete' in r, r

    def test_var_sub_put_method(self):
        r = self.app.put('/rest3/1/3/3/subrest/1')
        assert 'subrest put' in r, r
        r = self.app.put('/rest3/1/3/subrest/1')
        assert 'subrest put' in r, r
        r = self.app.put('/rest3/subrest/1')
        assert 'subrest put' in r, r

    def test_var_sub_put_hack_method(self):
        r = self.app.post('/rest3/1/3/3/subrest/1?_method=PUT')
        assert 'subrest put' in r, r
        r = self.app.post('/rest3/1/3/subrest/1?_method=put')
        assert 'subrest put' in r, r
        r = self.app.post('/rest3/subrest/1?_method=put')
        assert 'subrest put' in r, r

    def test_var_sub_var_delete_method(self):
        r = self.app.delete('/rest3/1/3/3/vsubrest/1')
        assert 'subrest post delete' in r, r
        r = self.app.delete('/rest3/1/3/vsubrest/1')
        assert 'subrest post delete' in r, r
        r = self.app.delete('/rest3/vsubrest/')
        assert 'subrest post delete' in r, r

    def test_var_sub_delete_var_hack_method(self):
        r = self.app.post('/rest3/1/3/3/vsubrest/1?_method=DELETE')
        assert 'subrest post delete' in r, r
        r = self.app.post('/rest3/1/3/vsubrest/1?_method=delete')
        assert 'subrest post delete' in r, r
        r = self.app.post('/rest3/vsubrest?_method=delete')
        assert 'subrest post delete' in r, r

    def test_var_sub_var_put_hack_method(self):
        r = self.app.post('/rest3/1/3/3/vsubrest/1?_method=PUT')
        assert 'subrest put' in r, r
        r = self.app.post('/rest3/1/3/vsubrest/1/a?_method=put')
        assert 'subrest put' in r, r
        r = self.app.post('/rest3/vsubrest/?_method=put')
        assert 'subrest put' in r, r

    def test_var_sub_delete_hack_method(self):
        r = self.app.post('/rest3/1/3/3/subrest/1?_method=DELETE')
        assert 'subrest post delete' in r, r
        r = self.app.post('/rest3/1/3/subrest/1?_method=delete')
        assert 'subrest post delete' in r, r
        r = self.app.post('/rest3/subrest/1?_method=delete')
        assert 'subrest post delete' in r, r

    def test_sub_new(self):
        r = self.app.get('/rest2/1/subrest/new')
        assert 'subrest new' in r, r

    def test_sub_edit(self):
        r = self.app.get('/rest2/1/subrest/1/edit')
        assert 'subrest edit' in r, r

    def test_sub_post(self):
        r = self.app.post('/rest2/1/subrest/')
        assert 'subrest post' in r, r

    def test_sub_put(self):
        r = self.app.put('/rest2/1/subrest/2')
        assert 'subrest put' in r, r

    def test_sub_post_opt(self):
        r = self.app.post('/rest2/1/optsubrest/1')
        assert 'subrest post' in r, r

    def test_sub_put_opt(self):
        r = self.app.put('/rest2/1/optsubrest/1')
        assert 'subrest put' in r, r

    def test_sub_put_opt_hack(self):
        r = self.app.post('/rest2/1/optsubrest/1?_method=PUT')
        assert 'subrest put' in r, r

    def test_sub_delete_opt_hack(self):
        r = self.app.post('/rest2/1/optsubrest/1?_method=DELETE')
        assert 'subrest ' in r, r

    def test_put_post_req(self):
        r = self.app.post('/rest2/reqsubrest', params={'something':'required'})
        assert 'subrest post' in r, r

    def test_sub_put_req(self):
        r = self.app.post('/rest2/reqsubrest', params={'_method':'PUT', 'something':'required'})
        assert 'subrest put' in r, r

    def test_sub_post_req_bad(self):
        r = self.app.post('/rest2/reqsubrest',)
        assert "['rest2', 'reqsubrest']" in r, r

    def test_sub_delete_hack(self):
        r = self.app.post('/rest2/1/subrest/2?_method=DELETE')
        assert 'subrest post delete' in r, r

    def test_sub_get_delete(self):
        r = self.app.get('/rest2/1/subrest/2/delete')
        assert 'subrest get delete' in r, r

    def test_sub_post_delete(self):
        r = self.app.delete('/rest2/1/subrest/2')
        assert 'subrest post delete' in r, r

    def test_sub_get_fxn(self):
        r = self.app.get('/rest2/1/subrest/fxn')
        assert 'subrest fxn' in r, r

    def test_sub_post_fxn(self):
        r = self.app.post('/rest2/1/subrest/fxn')
        assert 'subrest fxn' in r, r



import logging
from .base import ApplicationWrapper
from ..configuration.utils import coerce_config
from ..support.converters import asbool

log = logging.getLogger(__name__)


class IdentityApplicationWrapper(ApplicationWrapper):
    """Provides user identity when authentication is enabled.

    The repoze.who provided identity takes precedence over the identity
    provided by IdentityApplicationWrapper if available.

    Supported options which can be provided by config are:
        - ``sa_auth.authmetadata``: The TGAuthMetadata object that should be used to retrieve identity metadata.
        - ``identity.enabled``: Enable the Identity Application Wrapper. By default enabled if authmetadata available.
        - ``identity.allow_missing_user``: Whenever the identity should be discarded or not when the authmetadata is unable to find an user.

    """
    def __init__(self, handler, config):
        super(IdentityApplicationWrapper, self).__init__(handler, config)

        options = {
            'enabled': True,
            'allow_missing_user': True,
            'authmetadata': config.get('sa_auth',  {}).get('authmetadata'),
        }
        options.update(coerce_config(config, 'identity.',  {
            'enabled': asbool,
            'allow_missing_user': asbool
        }))

        self.enabled = options['enabled'] and options['authmetadata'] is not None
        self.options = options
        self.tgmdprovider = options['authmetadata']
        log.debug('Identity enabled: %s -> %s', self.enabled, self.options)

    @property
    def injected(self):
        return self.enabled

    def __call__(self, controller, environ, context):
        identity = environ.get('repoze.who.identity')
        if identity is None:
            context.request.identity = None
            return self.next_handler(controller, environ, context)

        req_identity = {}

        # Get the userid retrieved by repoze.who Authenticator
        userid = identity['repoze.who.userid']
        if userid is not None:
            # Finding the user, groups and permissions:
            identity['user'] = identity_user = self.tgmdprovider.get_user(identity, userid)

            if identity_user:
                identity['groups'] = self.tgmdprovider.get_groups(identity, userid)
                identity['permissions'] = self.tgmdprovider.get_permissions(identity, userid)
            else:
                identity['groups'] = identity['permissions'] = []

            req_identity = Identity()
            req_identity.update(identity)
            req_identity['repoze.what.userid'] = userid

            if req_identity.get('user') is None and not self.options['allow_missing_user']:
                req_identity = {}

        # Add identity to request with repoze.who/what compatibility
        context.request.identity = req_identity
        environ['repoze.who.identity'] = req_identity
        environ['repoze.what.credentials'] = req_identity

        return self.next_handler(controller, environ, context)


class Identity(dict):
    """dict subclass: prevent members from being rendered during print.

    Took as is from repoze.who.
    """
    def __repr__(self):
        return '<TurboGears Identity (hidden, dict-like) at %s>' % id(self)
    __str__ = __repr__


# -*- coding: utf-8 -*-

import os, shutil
from unittest import TestCase
from tg.appwrappers.caching import CacheApplicationWrapper
from tg.appwrappers.errorpage import ErrorPageApplicationWrapper
from tg.appwrappers.i18n import I18NApplicationWrapper
from tg.appwrappers.identity import IdentityApplicationWrapper
from tg.appwrappers.session import SessionApplicationWrapper
from tg.configuration.utils import DependenciesList

try:
    from xmlrpclib import loads, dumps
except ImportError:
    from xmlrpc.client import loads, dumps
import warnings

from tg.support.registry import Registry, RegistryManager

from webtest import TestApp

import tg
from tg import tmpl_context, request_local
from tg.configuration import milestones

from tg.wsgiapp import TemplateContext, TGApp, RequestLocals
from tg.controllers import TGController

from .test_stack.baseutils import ControllerWrap, default_config

data_dir = os.path.dirname(os.path.abspath(__file__))
session_dir = os.path.join(data_dir, 'session')
cache_dir = os.path.join(data_dir, 'cache')

def setup_session_dir():
    if not os.path.exists(session_dir):
        os.makedirs(session_dir)

def teardown_session_dir():
    shutil.rmtree(session_dir, ignore_errors=True)

def make_app(controller_klass=None, environ=None, config_options=None, with_errors=False):
    """Creates a `TestApp` instance."""
    if controller_klass is None:
        controller_klass = TGController

    tg.config['renderers'] = default_config['renderers']
    tg.config['rendering_engines_options'] = default_config['rendering_engines_options']

    config = default_config.copy()
    config['application_wrappers'] = DependenciesList(
        I18NApplicationWrapper,
        IdentityApplicationWrapper,
        CacheApplicationWrapper,
        SessionApplicationWrapper
    )

    if with_errors:
        config['errorpage.enabled'] = True
        config['errorpage.status_codes'] = [403, 404]
        config['application_wrappers'].add(ErrorPageApplicationWrapper)

    config['session.enabled'] = True
    config['session.data_dir'] = session_dir
    config['cache.enabled'] = True
    config['cache.cache_dir'] = cache_dir

    if config_options is not None:
        config.update(config_options)

    app = TGApp(config=config)
    app.controller_classes['root'] = ControllerWrap(controller_klass)

    app = RegistryManager(app)
    return TestApp(app)


class TestWSGIController(TestCase):
    def setUp(self):
        tmpl_options = {}
        tmpl_options['genshi.search_path'] = ['tests']

        self._tgl = RequestLocals()
        self._tgl.tmpl_context = TemplateContext()
        request_local.context._push_object(self._tgl)

        # Mark configuration milestones as passed as
        # test sets up a fake configuration
        milestones._reach_all()

        warnings.simplefilter("ignore")
        tg.config.push_process_config(default_config)
        warnings.resetwarnings()
        setup_session_dir()

    def tearDown(self):
        request_local.context._pop_object(self._tgl)
        tg.config.pop_process_config()
        teardown_session_dir()

        # Reset milestones
        milestones._reset_all()

    def get_response(self, **kargs):
        url = kargs.pop('_url', '/')
        return self.app.get(url, extra_environ=self.environ)

    def post_response(self, **kargs):
        url = kargs.pop('_url', '/')

        return self.app.post(url, extra_environ=self.environ, params=kargs)




import inspect
from collections import deque

import itertools

from .milestones import config_ready


class TGConfigError(Exception):pass


def coerce_options(options, converters):
    """Convert some configuration options to expected types.

    To replace given options with the converted values
    in a dictionary you might do::

        conf.update(coerce_options(conf, {
            'debug': asbool,
            'serve_static': asbool,
            'auto_reload_templates': asbool
        }))
    """
    converted_options = {}
    for option, converter in converters.items():
        if option in options:
            converted_options[option] = converter(options[option])
    return converted_options


def coerce_config(configuration, prefix, converters):
    """Extracts a set of options with a common prefix and converts them.

    To extract all options starting with ``trace_errors.`` from
    the ``conf`` dictionary and conver them::

        trace_errors_config = coerce_config(conf, 'trace_errors.', {
            'smtp_use_tls': asbool,
            'dump_request_size': asint,
            'dump_request': asbool,
            'dump_local_frames': asbool,
            'dump_local_frames_count': asint
        })
    """

    options = dict((key[len(prefix):], configuration[key])
                    for key in configuration if key.startswith(prefix))
    options.update(coerce_options(options, converters))
    return options

def get_partial_dict(prefix, dictionary, container_type=dict):
    """Given a dictionary and a prefix, return a Bunch, with just items
    that start with prefix

    The returned dictionary will have 'prefix.' stripped so::

        get_partial_dict('prefix', {'prefix.xyz':1, 'prefix.zyx':2, 'xy':3})

    would return::

        {'xyz':1,'zyx':2}
    """

    match = prefix + "."
    n = len(match)

    new_dict = container_type(((key[n:], dictionary[key])
                                for key in dictionary
                                if key.startswith(match)))
    if new_dict:
        return new_dict
    else:
        raise AttributeError(prefix)


class GlobalConfigurable(object):
    """Defines a configurable TurboGears object with a global default instance.

    GlobalConfigurable are objects which the user can create multiple instances to use
    in its own application or third party module, but for which TurboGears provides
    a default instance.

    Common examples are ``tg.flash`` and the default JSON encoder for which
    TurboGears provides default instances of ``.TGFlash`` and ``.JSONEncoder`` classes
    but users can create their own.

    While user created versions are configured calling the :meth:`.GlobalConfigurable.configure`
    method, global versions are configured by :class:`.AppConfig` which configures them when
    ``config_ready`` milestone is reached.

    """
    CONFIG_NAMESPACE = None
    CONFIG_OPTIONS = {}

    def configure(self, **options):
        """Expected to be implemented by each object to proceed with actualy configuration.

        Configure method will receive all the options whose name starts with ``CONFIG_NAMESPACE``
        (example ``json.isodates`` has ``json.`` namespace).

        If ``CONFIG_OPTIONS`` is specified options values will be converted with
        :func:`coerce_config` passing ``CONFIG_OPTIONS`` as the ``converters`` dictionary.

        """
        raise NotImplementedError('GlobalConfigurable objects must implement a configure method')

    @classmethod
    def create_global(cls):
        """Creates a global instance which configuration will be bound to :class:`.AppConfig`."""
        if cls.CONFIG_NAMESPACE is None:
            raise TGConfigError('Must specify a CONFIG_NAMESPACE attribute in class for the'
                                'namespace used by all configuration options.')

        obj = cls()
        config_ready.register(obj._load_config, persist_on_reset=True)
        return obj

    def _load_config(self):
        from tg.configuration import config
        self.configure(**coerce_config(config, self.CONFIG_NAMESPACE,  self.CONFIG_OPTIONS))


class DependenciesList(object):
    """Manages a list of entries which might depend one from the other.

    This powers :meth:`.AppConfig.register_wrapper` and other features in
    TurboGears2, making possible to register the wrappers right after
    other wrappers or at the end of the wrappers chain.

    .. note:: This is highly inefficient as it is only meant to run at configuration time,
              a new implementation will probably be provided based on heapq in the future.
    """
    #: Those are the heads of the dependencies tree
    #:  - ``False`` means before everything else
    #:  - ``None`` means in the middle.
    #:  - ``True`` means after everything else.
    DEPENDENCY_HEADS = (False, None, True)

    def __init__(self, *entries):
        self._dependencies = {}
        self._ordered_elements = []
        self._inserted_keys = []

        for entry in entries:
            self.add(entry)

    def add(self, entry, key=None, after=None):
        """Adds an entry to the dependencies list.

        :param entry: Entry that must be added to the list.
        :param str|type|None key: An identifier of the object being inserted.
                                  This is used by later insertions as ``after`` argument
                                  to specify after which object the new one should be inserted.
        :param str|type|None|False|True after: After which element this one should be inserted.
                                               This is the ``key`` of a previously inserted item.
                                               In case no item with ``key`` has been inserted, the
                                               entry will be inserted in normal order of insertion.
                                               Also accepts one of
                                               :attr:`.DependenciesList.DEPENDENCY_HEADS` as key
                                               to add entries at begin or end of the list.
        """
        if key is None:
            if inspect.isclass(entry):
                key = entry.__name__
            else:
                # Inserting an object without a key would lead to unexpected ordering.
                # we cannot use the object class as the key would not be unique across
                # different instances.
                raise ValueError('Inserting instances without a key is not allowed')

        if after not in self.DEPENDENCY_HEADS and not isinstance(after, str):
            if inspect.isclass(after):
                after = after.__name__
            else:
                raise ValueError('after parameter must be a string, a class or a special value')

        self._inserted_keys.append(key)
        self._dependencies.setdefault(after, []).append((key, entry))
        self._resolve_ordering()

    def __repr__(self):
        return '<DependenciesList %s>' % [x[0] for x in self._ordered_elements]

    def __iter__(self):
        return iter(self._ordered_elements)

    def values(self):
        """Returns all the inserted values without their key as a generator"""
        return (x[1]for x in self._ordered_elements)

    def replace(self, key, newvalue):
        """Replaces entry associated to key with a new one.

        :param newvalue: Entry that must replace the previous value.
        :param str|type key: An identifier of the object being inserted.
        """
        if not isinstance(key, str):
            if inspect.isclass(key):
                key = key.__name__
            else:
                raise ValueError('key parameter must be a string or a class')

        for entries in self._dependencies.values():
            for idx, value in enumerate(entries):
                entry_key, entry_value = value
                if entry_key == key:
                    entries[idx] = (entry_key, newvalue)

        self._resolve_ordering()

    def _resolve_ordering(self):
        ordered_elements = []

        existing_dependencies = set(self._inserted_keys) | set(self.DEPENDENCY_HEADS)
        dependencies_without_heads = set(self._dependencies.keys()) - set(self.DEPENDENCY_HEADS)

        # All entries that depend on a missing entry are converted
        # to depend from None so they end up being in the middle.
        dependencies = {}
        for dependency in itertools.chain(self.DEPENDENCY_HEADS, dependencies_without_heads):
            entries = self._dependencies.get(dependency, [])
            if dependency not in existing_dependencies:
                dependency = None
            dependencies.setdefault(dependency, []).extend(entries)

        # Resolve the dependencies and generate the ordered elements.
        visit_queue = deque((head, head) for head in self.DEPENDENCY_HEADS)
        while visit_queue:
            current_key, current_obj = visit_queue.popleft()
            if current_key not in self.DEPENDENCY_HEADS:
                ordered_elements.append((current_key, current_obj))

            element_dependencies = dependencies.pop(current_key, [])
            visit_queue.extendleft(reversed(element_dependencies))

        self._ordered_elements = ordered_elements



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json

import tg
from tg.controllers import TGController
from tg.decorators import expose, validate, https, variable_decode, with_trailing_slash, \
    without_trailing_slash, with_engine
from tg import expose, redirect, config
from tg.controllers import TGController
from tg import dispatched_controller
from nose.tools import eq_
from tests.test_validation import validators
from tg._compat import unicode_text

from paste.deploy.converters import asbool

class NestedSubController(TGController):
    @expose()
    def index(self):
        return '-'.join((self.mount_point, dispatched_controller().mount_point))

    @expose()
    def hitme(self):
        return '*'.join((self.mount_point, dispatched_controller().mount_point))

    @expose()
    def _lookup(self, *args):
        lookup = LookupController()
        return lookup, args

class SubController(TGController):
    nested = NestedSubController()
    
    @expose()
    def foo(self,):
        return 'sub_foo'

    @expose()
    def index(self):
        return 'sub index'

    @expose()
    def _default(self, *args):
        return ("recieved the following args (from the url): %s" %list(args))

    @expose()
    def redirect_me(self, target, **kw):
        tg.redirect(target, **kw)

    @expose()
    def redirect_sub(self):
        tg.redirect('index')

    @expose()
    def redirect_list(self):
        tg.redirect(["/sub2", "list"])

    @expose()
    def hello(self, name):
        return "Why HELLO! " + name

    @expose()
    def hitme(self):
        return '@'.join((self.mount_point, dispatched_controller().mount_point))

class LookupController(TGController):
    nested = NestedSubController()
    
    @expose()
    def findme(self, *args, **kw):
        return 'got to lookup'

    @expose()
    def hiddenhitme(self, *args, **kw):
        return ' '.join((self.mount_point, dispatched_controller().mount_point))

class SubController2(object):
    @expose()
    def index(self):
        tg.redirect('list')

    @expose()
    def list(self, **kw):
        return "hello list"

    @expose()
    def _lookup(self, *args):
        lookup = LookupController()
        return lookup, args

class RootController(TGController):
    @expose()
    def index(self, **kwargs):
        return 'hello world'

    @expose()
    def _default(self, remainder):
        return "Main Default Page called for url /%s"%remainder

    @expose()
    def return_something(self):
        return 5

    @expose()
    def return_none(self):
        return None

    @expose()
    def return_modified_response(self):
        tg.response.status_int = 201
        tg.response.content_type = 'application/json'
        tg.response.charset = 'utf-8'
        tg.response.body = json.dumps({'text': 'Hello World'}).encode('utf-8')
        return tg.response

    @expose()
    def feed(self, feed=None):
        return feed

    sub = SubController()
    sub2 = SubController2()

    @expose()
    def redirect_me(self, target, **kw):
        tg.redirect(target, kw)

    @expose()
    def hello(self, name, silly=None):
        return "Hello " + name

    @expose()
    def redirect_cookie(self, name):
        tg.response.set_cookie('name', name)
        tg.redirect('/hello_cookie')

    @expose()
    def hello_cookie(self):
        return "Hello " + tg.request.cookies['name']

    @expose()
    def flash_redirect(self):
        tg.flash("Wow, <br/>flash!")
        tg.redirect("/flash_after_redirect")

    @expose()
    def flash_render(self, using_js=False, with_message=True):
        if asbool(with_message):
            tg.flash('JS <br/>Flash')

        return tg.flash.render('flash', asbool(using_js))

    @expose()
    def bigflash_redirect(self):
        tg.flash('x' * 5000)
        tg.redirect('/flash_after_redirect')

    @expose()
    def flash_unicode(self):
        tg.flash("Привет, мир!")
        tg.redirect("/flash_after_redirect")

    @expose()
    def flash_after_redirect(self):
        return tg.get_flash()

    @expose()
    def flash_status(self):
        return tg.get_status()

    @expose()
    def flash_no_redirect(self):
        tg.flash("Wow, flash!")
        return tg.get_flash()

    @expose('json')
    @validate(validators={"some_int": validators.Int()})
    def validated_int(self, some_int):
        assert isinstance(some_int, int)
        return dict(response=some_int)

    @expose('json')
    @validate(validators={"a":validators.Int()})
    def validated_and_unvalidated(self, a, b):
        assert isinstance(a, int)
        assert isinstance(b, unicode_text)
        return dict(int=a,str=b)

    @expose()
    @expose('json')
    def stacked_expose(self, tg_format=None):
        return dict(got_json=True)

    @expose(content_type='image/png')
    def custom_content_type(self):
        return b'PNG'

    @expose()
    def custom_content_type2(self):
        tg.response.content_type = 'image/png'
        return b'PNG2'

    @expose(content_type='text/plain')
    def custom_content_text_plain_type(self):
        return 'a<br/>bx'

    @expose()
    def check_params(self, *args, **kwargs):
        if not args and not kwargs:
            return "None recieved"
        else:
            return "Controler recieved: %s, %s" %(args, kwargs)

    @expose()
    def test_url_sop(self):
        from tg import url
        eq_('/foo', url('/foo'))


        u = url("/foo", params=dict(bar=1, baz=2))
        assert u in \
                ["/foo?bar=1&baz=2", "/foo?baz=2&bar=1"], u

    @https
    @expose()
    def test_https(self, **kw):
        return ''

    @expose('json')
    @variable_decode
    def test_vardec(self, **kw):
        return kw

    @expose('mako:echo.mak')
    def echo(self):
        return dict()

    @expose()
    @with_trailing_slash
    def with_tslash(self):
        return 'HI'

    @expose()
    @without_trailing_slash
    def without_tslash(self):
        return 'HI'

    @expose()
    @with_engine('mainslave')
    def onmaster_without_params(self, **kw):
        return '%s-%s' % (tg.request._tg_force_sqla_engine, kw)

    @expose()
    @with_engine('mainslave', master_params=['first'])
    def onmaster_withlist(self, **kw):
        return '%s-%s' % (tg.request._tg_force_sqla_engine, kw)

    @expose()
    @with_engine('mainslave', master_params={'first':True, 'second':False})
    def onmaster(self, **kw):
        return '%s-%s' % (tg.request._tg_force_sqla_engine, kw)


# -*- coding: utf-8 -*-
from datetime import datetime, timedelta

import tg
from tg.util import *
from tg.configuration.utils import get_partial_dict
from nose.tools import eq_, raises, assert_raises
import os
from tg.controllers.util import *
from tg.util.dates import get_fixed_timezone, utctz, parse_datetime
from tg.util.files import safe_filename
from tg.util.html import script_json_encode
from tg.util.misc import unless
from tg.util.webtest import test_context
from tg.validation import Convert, TGValidationError
from tg.wsgiapp import TemplateContext, AttribSafeTemplateContext

import tg._compat
from tg._compat import u_

path = None
def setup():
    global path
    path = os.curdir
    os.chdir(os.path.abspath(os.path.dirname(os.path.dirname(tg.__file__))))

def teardown():
    global path
    os.chdir(path)

def test_get_partial_dict():
    eq_(get_partial_dict('prefix', {'prefix.xyz':1, 'prefix.zyx':2, 'xy':3}),
        {'xyz':1,'zyx':2})

def test_compat_im_class():
    class FakeClass(object):
        def method(self):
            pass

    def func():
        pass

    o = FakeClass()
    assert tg._compat.im_class(o.method) == FakeClass
    assert tg._compat.im_class(func) == None


class TestUrlMethod(object):
    def test_url_unicode(self):
        with test_context(None, '/'):
            res = url('.', {'p1':u_('v1')})
            assert res == '.?p1=v1'

    def test_url_unicode_nonascii(self):
        with test_context(None, '/'):
            res = url('.', {'p1':u_('àèìòù')})
            assert res == '.?p1=%C3%A0%C3%A8%C3%AC%C3%B2%C3%B9'

    def test_url_nonstring(self):
        with test_context(None, '/'):
            res = url('.', {'p1':1})
            assert res == '.?p1=1'

    def test_url_object(self):
        class Object(object):
            def __str__(self):
                return 'aeiou'

        with test_context(None, '/'):
            res = url('.', {'p1': Object()})
            assert res == '.?p1=aeiou'

    def test_url_object_unicodeerror(self):
        class Object(object):
            def __str__(self):
                return u_('àèìòù')

        with test_context(None, '/'):
            res = url('.', {'p1': Object()})
            assert res == '.?p1=%C3%A0%C3%A8%C3%AC%C3%B2%C3%B9'

    def test_url_object_exception(self):
        class SubException(Exception):
            def __str__(self):
                return u_('àèìòù')

        with test_context(None, '/'):
            res = url('.', {'p1': SubException('a', 'b', 'c')})
            assert res == '.?p1=a+b+c', res

    def test_url_https(self):
        with test_context(None, '/index'):
            res = url('/users', qualified=True, scheme='https')
            assert res == 'https://localhost/users', res


class TestBunch(object):
    def test_add_entry(self):
        d = Bunch()
        d['test.value'] = 5
        assert d.test.value == 5

    def test_del_entry(self):
        d = Bunch()
        d['test_value'] = 5
        del d.test_value
        assert not list(d.keys())

    @raises(AttributeError)
    def test_del_entry_fail(self):
        d = Bunch()
        del d.not_existing


class TestDottedNameFinder(object):
    @raises(DottedFileLocatorError)
    def test_non_python_package(self):
        DottedFileNameFinder().get_dotted_filename('this.is.not.a.python.package')

    def test_local_file(self):
        abspath = os.path.abspath('this_should_be_my_template')
        assert DottedFileNameFinder().get_dotted_filename('this_should_be_my_template') == abspath

    def test_local_file_utility_method(self):
        abspath = os.path.abspath('this_should_be_my_template')
        assert DottedFileNameFinder.lookup('this_should_be_my_template') == abspath

    def test_local_file_returns_absolute_path(self):
        assert os.path.isabs(DottedFileNameFinder.lookup('this_should_be_my_template'))

    def test_load_from_zipped_egg(self):
        import sys
        eggfile = os.path.join(os.path.dirname(__file__), 'fixtures', 'fakepackage.zip')
        sys.path.append(eggfile)

        tmplf = DottedFileNameFinder().get_dotted_filename('fakepackage.test_template',
                                                           template_extension='.xhtml')
        with open(tmplf) as t:
            template = t.read()
            assert template == '<p>Your application is now running</p>', template


class TestLazyString(object):
    def test_lazy_string_to_str(self):
        l = LazyString(lambda: 'HI')
        assert str(l) == 'HI'

    def test_lazy_string_to_mod(self):
        l = LazyString(lambda: '%s')
        assert (l % 'HI') == 'HI'

    def test_lazy_string_format(self):
        l = LazyString(lambda: '{0}')
        lf = l.format('HI')
        assert lf == 'HI', lf

    def test_lazy_string_with_genshi(self):
        # See https://github.com/TurboGears/tg2/pull/68
        from genshi.template.markup import MarkupTemplate
        markup = """<b xmlns:py="http://genshi.edgewall.org/">${foo}</b>"""
        template = MarkupTemplate(markup)
        stream = template.generate(foo=LazyString(lambda: "bar"))
        output = str(stream)  # Contains only ascii char, so it should cast fine on both py2 and py3
        assert output == '<b>bar</b>', output


class TestAttribSafeContextObj(object):
    def setup(self):
        self.c = AttribSafeTemplateContext()

    def test_attribute_default_value(self):
        assert self.c.something == ''

        self.c.something = 'HELLO'
        assert self.c.something == 'HELLO'

        assert self.c.more == ''

def test_tmpl_context_long_entry():
    c = TemplateContext()
    c.something = '3'*300
    assert len(str(c)) < 300



class TestDatesUtils(object):
    def test_get_fixed_timezone_seconds(self):
        delta = get_fixed_timezone(0.5).utcoffset(None)
        assert delta.seconds == 0

    def test_get_fixed_timezone_minutes(self):
        delta = get_fixed_timezone(1).utcoffset(None)
        assert delta.seconds == 60

    def test_get_fixed_timezone_hours(self):
        delta = get_fixed_timezone(60).utcoffset(None)
        assert delta.seconds == 3600

    def test_get_fixed_timezone_seconds_td(self):
        delta = get_fixed_timezone(timedelta(seconds=30)).utcoffset(None)
        assert delta.seconds == 0

    def test_get_fixed_timezone_minutes_td(self):
        delta = get_fixed_timezone(timedelta(minutes=1)).utcoffset(None)
        assert delta.seconds == 60

    def test_get_fixed_timezone_hours_td(self):
        delta = get_fixed_timezone(timedelta(hours=1)).utcoffset(None)
        assert delta.seconds == 3600

    def test_get_fixed_timezone_usage(self):
        utcnow = datetime.utcnow()

        uktz = get_fixed_timezone(60)
        uknow = (utcnow + timedelta(hours=1)).replace(tzinfo=uktz)

        naiveuk = uknow.astimezone(utctz).replace(tzinfo=None)
        assert utcnow == naiveuk, (utcnow, naiveuk)

    def test_get_fixed_timezone_name(self):
        uktz = get_fixed_timezone(60)
        uktz_name = uktz.tzname(None)
        assert uktz_name == '+0100', uktz_name
        assert repr(uktz) == '<+0100>', repr(uktz)

        assert utctz.tzname(None) == 'UTC', utctz.tzname(None)
        assert repr(utctz) == '<UTC>', repr(utctz)

    def test_get_fixed_timezone_unknowndst(self):
        uktz = get_fixed_timezone(60)
        assert uktz.dst(None).seconds == 0

    def test_parse_datetime_tz(self):
        dt = parse_datetime('1997-07-16T19:20:30.45+01:00')
        assert dt.tzname() == '+0100', dt

        expected_dt = datetime(1997, 7, 16, 19, 20, 30, 450000)
        naive_dt = dt.replace(tzinfo=None)
        assert naive_dt == expected_dt, naive_dt

    def test_parse_datetime_utc(self):
        dt = parse_datetime('1997-07-16T19:20:30.45Z')
        assert dt.tzname() == 'UTC', dt

        expected_dt = datetime(1997, 7, 16, 19, 20, 30, 450000).replace(tzinfo=utctz)
        assert dt == expected_dt, dt

    def test_parse_datetime_negativetz(self):
        dt = parse_datetime('1997-07-16T19:20:30.45-01:00')
        assert dt.tzname() == '-0100', dt

        expected_dt = datetime(1997, 7, 16, 19, 20, 30, 450000)
        naive_dt = dt.replace(tzinfo=None)
        assert naive_dt == expected_dt, naive_dt

    @raises(ValueError)
    def test_parse_datetime_invalid_format(self):
        parse_datetime('1997@07@16T19:20:30.45+01:00')


class TestHtmlUtils(object):
    def test_script_json_encode(self):
        rv = script_json_encode('</script>')
        assert rv == u_('"\\u003c/script\\u003e"')
        rv = script_json_encode("<\0/script>")
        assert rv == '"\\u003c\\u0000/script\\u003e"'
        rv = script_json_encode("<!--<script>")
        assert rv == '"\\u003c!--\\u003cscript\\u003e"'
        rv = script_json_encode("&")
        assert rv == '"\\u0026"'
        rv = script_json_encode("\'")
        assert rv == '"\\u0027"'
        rv = "<a ng-data='%s'></a>" % script_json_encode({'x': ["foo", "bar", "baz'"]})
        assert rv == '<a ng-data=\'{"x": ["foo", "bar", "baz\\u0027"]}\'></a>'

    def test_script_json_encode_array(self):
        rv = "<a ng-data='%s'></a>" % script_json_encode(['1', 2, 5])
        assert rv == '<a ng-data=\'["1", 2, 5]\'></a>', rv


class TestFilesUtils(object):
    def test_safe_filename(self):
        assert safe_filename('My cool movie.mov') == 'My_cool_movie.mov'
        assert safe_filename('../../../etc/passwd') == 'etc_passwd'
        assert safe_filename(u_('i contain cool ümläuts.txt')) == 'i_contain_cool_umlauts.txt'


class TestWebTestUtilities(object):
    def test_test_context(self):
        with test_context(None):
            test_url = url('/test')
            assert test_url == '/test', test_url

        try:
            url('/test')
        except:
            pass
        else:
            assert False, 'Should have raised exception...'

    def test_test_context_broken_app(self):
        from webtest import TestApp
        from tg import AppConfig, config
        from tg.request_local import context

        app = TestApp(AppConfig(
            minimal=True,
            root_controller=None
        ).make_wsgi_app())

        try:
            with test_context(app):
                raise RuntimeError('This is an error')
        except RuntimeError:
            pass
        else:
            assert False, 'Should have raised RuntimeError...'

        with test_context(app):
            config._pop_object()
        # Check that context got cleaned up even though config caused an exception
        assert not context._object_stack()

        with test_context(app):
            context._pop_object()
        # Check that config got cleaned up even though context caused an exception
        assert not config._object_stack()


class TestMiscUtils(object):
    def test_unless(self):
        not5 = unless(lambda x: x % 5, 0)
        assert not5(6) == 1

        assert_raises(ValueError, not5, 10)

    def test_unless_sqla(self):
        from sqlalchemy import (MetaData, Table, Column, Integer, String)
        from sqlalchemy.orm import create_session, mapper

        metadata = MetaData('sqlite:///:memory:')
        testtable = Table('test1', metadata,
            Column('id', Integer, primary_key=True),
            Column('val', String(8)))
        metadata.create_all()

        class Test(object):
            pass
        mapper(Test, testtable)

        testtable.insert().execute({'id': 1, 'val': 'bob'})
        testtable.insert().execute({'id': 2, 'val': 'bobby'})
        testtable.insert().execute({'id': 3, 'val': 'alberto'})

        sess = create_session()
        getunless = unless(sess.query(Test).get)

        x = getunless(1)
        assert x.val == 'bob', x

        x = getunless(2)
        assert x.val == 'bobby', x

        assert_raises(ValueError, getunless, 5)
        assert_raises(TGValidationError, Convert(getunless).to_python, '5')

        x = Convert(getunless).to_python('1')
        assert x.val == 'bob', x


import logging
import warnings
from ..support.converters import asbool
from .base import ApplicationWrapper

try:
    from beaker.session import Session, SessionObject
except ImportError:  # pragma: no cover
    SessionObject = None


log = logging.getLogger(__name__)


class SessionApplicationWrapper(ApplicationWrapper):
    """Provides the Session Support

    The Session Application Wrapper will make a lazy session instance
    available every request under the ``environ['beaker.session']`` key and
    inside TurboGears context as ``session``.

    Supported options which can be provided by config are:
        - ``session.enabled``: Whenever sessions are enabled or not.
        - Beaker Options prefixed with ``session.``, see
          https://beaker.readthedocs.org/en/latest/configuration.html#session-options

    """
    def __init__(self, handler, config):
        super(SessionApplicationWrapper, self).__init__(handler, config)

        if SessionObject is None:  # pragma: no cover
            log.debug('Beaker not available, session disabled')
            self.enabled = False
            return

        # Load up the default params
        self.options = dict(invalidate_corrupt=True, type=None,
                            data_dir=None, key='beaker.session.id',
                            timeout=None, secret=None, log_file=None)

        # Pull out any config args meant for beaker session. if there are any
        for key, val in config.items():
            if key.startswith('beaker.session.'):
                warnings.warn('Session options should start with session. '
                              'instead of baker.session.', DeprecationWarning, 2)
                self.options[key[15:]] = val
            elif key.startswith('session.'):
                self.options[key[8:]] = val

        # Coerce and validate session params
        from beaker.util import coerce_session_params
        coerce_session_params(self.options)

        self.enabled = asbool(self.options.pop('enabled', True))

        log.debug('Sessions enabled: %s -> %s',
                  self.enabled, self.options)

    @property
    def injected(self):
        return self.enabled

    def __call__(self, controller, environ, context):
        context.session = session = SessionObject(environ, **self.options)
        environ['beaker.session'] = session
        environ['beaker.get_session'] = self._get_session

        if 'paste.testing_variables' in environ:
            environ['paste.testing_variables']['session'] = session

        response = self.next_handler(controller, environ, context)

        if session.accessed():
            session.persist()
            session_headers = session.__dict__['_headers']
            if session_headers['set_cookie']:
                cookie = session_headers['cookie_out']
                if cookie:
                    response.headers.extend((('Set-cookie', cookie),))

        return response

    def _get_session(self, session_id=None):
        return Session({}, session_id, use_cookies=False, **self.options)



# (c) 2005 Ben Bangert
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
from nose.tools import raises

from webtest import TestApp
from tg.support.registry import RegistryManager, StackedObjectProxy, DispatchingConfig
from tg.util import Bunch

regobj = StackedObjectProxy()
secondobj = StackedObjectProxy(default=dict(hi='people'))

def simpleapp(environ, start_response):
    status = '200 OK'
    response_headers = [('Content-type','text/plain')]
    start_response(status, response_headers)
    return ['Hello world!\n'.encode('utf-8')]

def simpleapp_withregistry(environ, start_response):
    status = '200 OK'
    response_headers = [('Content-type','text/plain')]
    start_response(status, response_headers)
    return [('Hello world!Value is %s\n' % regobj.keys()).encode('utf-8')]

def simpleapp_withregistry_default(environ, start_response):
    status = '200 OK'
    response_headers = [('Content-type','text/plain')]
    start_response(status, response_headers)
    return [('Hello world!Value is %s\n' % secondobj).encode('utf-8')]

class RegistryUsingApp(object):
    def __init__(self, var, value, raise_exc=False):
        self.var = var
        self.value = value
        self.raise_exc = raise_exc

    def __call__(self, environ, start_response):
        if 'paste.registry' in environ:
            environ['paste.registry'].register(self.var, self.value)
        if self.raise_exc:
            raise self.raise_exc
        status = '200 OK'
        response_headers = [('Content-type','text/plain')]
        start_response(status, response_headers)
        return [('Hello world!\nThe variable is %s' % str(regobj)).encode('utf-8')]

class RegistryUsingIteratorApp(object):
    def __init__(self, var, value):
        self.var = var
        self.value = value

    def __call__(self, environ, start_response):
        if 'paste.registry' in environ:
            environ['paste.registry'].register(self.var, self.value)
        status = '200 OK'
        response_headers = [('Content-type','text/plain')]
        start_response(status, response_headers)
        return iter([('Hello world!\nThe variable is %s' % str(regobj)).encode('utf-8')])

class RegistryMiddleMan(object):
    def __init__(self, app, var, value, depth):
        self.app = app
        self.var = var
        self.value = value
        self.depth = depth

    def __call__(self, environ, start_response):
        if 'paste.registry' in environ:
            environ['paste.registry'].register(self.var, self.value)
        app_response = [('\nInserted by middleware!\nInsertValue at depth \
            %s is %s' % (self.depth, str(regobj))).encode('utf-8')]
        app_iter = None
        app_iter = self.app(environ, start_response)
        if type(app_iter) in (list, tuple):
            app_response.extend(app_iter)
        else:
            response = []
            for line in app_iter:
                response.append(line)
            if hasattr(app_iter, 'close'):
                app_iter.close()
            app_response.extend(response)
        app_response.extend([('\nAppended by middleware!\nAppendValue at \
            depth %s is %s' % (self.depth, str(regobj))).encode('utf-8')])
        return app_response

def test_stacked_object_dir():
    regobj._push_object({'hi':'people'})
    try:
        values = dir(regobj)
        assert 'hi' in repr(regobj)
    finally:
        regobj._pop_object()

    assert '_current_obj' in values
    assert 'pop' in values
    assert 'items' in values

def test_stacked_object_dir_fail():
    values = dir(regobj)
    assert '_current_obj' in values

    assert repr(regobj).startswith('<tg.support.registry.StackedObjectProxy')

def test_stacked_object_callable():
    class Callable(object):
        def __call__(self, w):
            return w

    regobj._push_object(Callable())
    try:
        assert regobj('HI') == 'HI'
    finally:
        regobj._pop_object()

def test_stacked_object_common_actions():
    regobj._push_object(Bunch({'hi':'people'}))
    try:
        regobj['hi'] = 'val'
        assert regobj['hi'] == 'val'

        keys = []
        for k in regobj:
            keys.append(k)
        assert keys == ['hi'], keys

        assert len(regobj) == 1

        assert 'hi' in regobj

        assert bool(regobj) == True

        del regobj['hi']
        assert regobj.get('hi') is None

        regobj.someattr = 'val'
        assert regobj.someattr == 'val'

        del regobj.someattr
        assert getattr(regobj, 'someattr', None) is None
    finally:
        regobj._pop_object()

@raises(AssertionError)
def test_stacked_object_pop_something_else():
    o = Bunch({'hi':'people'})
    regobj._push_object(o)
    regobj._pop_object({'another':'object'})

@raises(AssertionError)
def test_stacked_object_pop_never_registered():
    regobj._pop_object()

def test_stacked_object_stack():
    so = StackedObjectProxy()

    assert(len(so._object_stack()) == 0)
    so._push_object({'hi':'people'})
    assert(len(so._object_stack()) == 1)
    so._pop_object()
    assert(len(so._object_stack()) == 0)

def test_stacked_object_preserve_empty():
    so = StackedObjectProxy()
    so._preserve_object()

    so._push_object({'hi':'people'})
    so._pop_object()
    so._preserve_object()

def test_stacked_object_preserved():
    so = StackedObjectProxy()
    assert not so._is_preserved

    so._push_object({'hi':'people'})
    assert not so._is_preserved

    so._pop_object()
    assert not so._is_preserved

    so._push_object({'hi':'people'})
    so._preserve_object()
    assert so._is_preserved
    so._pop_object()

def test_simple():
    app = TestApp(simpleapp)
    response = app.get('/')
    assert 'Hello world' in response

def test_solo_registry():
    obj = {'hi':'people'}
    wsgiapp = RegistryUsingApp(regobj, obj)
    wsgiapp = RegistryManager(wsgiapp)
    app = TestApp(wsgiapp)
    res = app.get('/')
    assert 'Hello world' in res
    assert 'The variable is' in res
    assert "{'hi': 'people'}" in res

@raises(TypeError)
def test_registry_no_object_error():
    app = TestApp(simpleapp_withregistry)
    app.get('/')

def test_with_default_object():
    app = TestApp(simpleapp_withregistry_default)
    res = app.get('/')
    assert 'Hello world' in res
    assert "Value is {'hi': 'people'}" in res

def test_double_registry():
    obj = {'hi':'people'}
    secondobj = {'bye':'friends'}
    wsgiapp = RegistryUsingApp(regobj, obj)
    wsgiapp = RegistryManager(wsgiapp)
    wsgiapp = RegistryMiddleMan(wsgiapp, regobj, secondobj, 0)
    wsgiapp = RegistryManager(wsgiapp)
    app = TestApp(wsgiapp)
    res = app.get('/')
    assert 'Hello world' in res
    assert 'The variable is' in res
    assert "{'hi': 'people'}" in res
    assert "InsertValue at depth 0 is {'bye': 'friends'}" in res
    assert "AppendValue at depth 0 is {'bye': 'friends'}" in res

def test_really_deep_registry():
    keylist = ['fred', 'wilma', 'barney', 'homer', 'marge', 'bart', 'lisa',
               'maggie']
    valuelist = range(0, len(keylist))
    obj = {'hi':'people'}
    wsgiapp = RegistryUsingApp(regobj, obj)
    wsgiapp = RegistryManager(wsgiapp)
    for depth in valuelist:
        newobj = {keylist[depth]: depth}
        wsgiapp = RegistryMiddleMan(wsgiapp, regobj, newobj, depth)
        wsgiapp = RegistryManager(wsgiapp)
    app = TestApp(wsgiapp)
    res = app.get('/')
    assert 'Hello world' in res
    assert 'The variable is' in res
    assert "{'hi': 'people'}" in res
    for depth in valuelist:
        assert "InsertValue at depth %s is {'%s': %s}" %\
               (depth, keylist[depth], depth) in res
    for depth in valuelist:
        assert "AppendValue at depth %s is {'%s': %s}" %\
               (depth, keylist[depth], depth) in res

def test_iterating_response():
    obj = {'hi':'people'}
    secondobj = {'bye':'friends'}
    wsgiapp = RegistryUsingIteratorApp(regobj, obj)
    wsgiapp = RegistryManager(wsgiapp)
    wsgiapp = RegistryMiddleMan(wsgiapp, regobj, secondobj, 0)
    wsgiapp = RegistryManager(wsgiapp)
    app = TestApp(wsgiapp)
    res = app.get('/')
    assert 'Hello world' in res
    assert 'The variable is' in res
    assert "{'hi': 'people'}" in res
    assert "InsertValue at depth 0 is {'bye': 'friends'}" in res
    assert "AppendValue at depth 0 is {'bye': 'friends'}" in res

def test_registry_streaming():
    def app(environ, start_response):
        environ['paste.registry'].register(regobj, {'hi':'people'})
        for i in range(10):
            yield str(i)
    rm = RegistryManager(app, streaming=True)

    environ = {}

    res = []
    for x in rm(environ, None):
        res.append(int(x))
        assert len(regobj._object_stack())

    assert len(res) == 10
    assert not(regobj._object_stack())

@raises(SystemError)
def test_registry_streaming_exception():
    def app(environ, start_response):
        environ['paste.registry'].register(regobj, {'hi':'people'})
        for i in range(10):
            if i == 5:
                raise SystemError('Woah!')
            else:
                yield str(i)
    app_with_rm = RegistryManager(app, streaming=True, preserve_exceptions=True)
    environ = {}
    try:
        for x in app_with_rm(environ, None):
            assert len(regobj._object_stack())
    except:
        #check the object got preserved due to exception
        assert regobj._object_stack()
        regobj._pop_object()
        raise


def test_registry_not_preserved_when_disabled():
    def app(environ, start_response):
        environ['paste.registry'].register(regobj, {'hi':'people'})
        environ['paste.registry'].preserve()
        return ['HI']

    app_with_rm = RegistryManager(app, streaming=False, preserve_exceptions=False)
    environ = {}

    app_with_rm(environ, None)

    # check the object are not preserved as preserve_exceptions is False
    assert not regobj._object_stack()


def test_registry_preserved_when_forcefuly_preserved():
    def app(environ, start_response):
        environ['paste.registry'].register(regobj, {'hi':'people'})
        environ['paste.registry'].preserve(force=True)
        return ['HI']

    app_with_rm = RegistryManager(app, streaming=False, preserve_exceptions=False)
    environ = {}

    app_with_rm(environ, None)

    # check the object are preserved as force=True was used
    assert regobj._object_stack()


def test_dispatch_config():
    conf = DispatchingConfig()
    conf.push_process_config({'key':'default'})
    conf.push_thread_config({'key':'value'})
    assert conf.current()['key'] == 'value'
    conf.pop_thread_config()
    assert conf.current()['key'] == 'default'

    try:
        conf.pop_process_config({'another':'one'})
        pop_failed = False
    except AssertionError:
        pop_failed = True
    assert pop_failed, 'It should have failed due to different config popped'

    try:
        conf.current()
        assert False, 'It should fail due to empty objects stack'
    except AttributeError:
        pass



"""This module contains the main TurboGears controller implementation."""

from crank.objectdispatcher import ObjectDispatcher

from tg.controllers.dispatcher import CoreDispatcher
from tg.controllers.decoratedcontroller import DecoratedController


class TGController(DecoratedController, CoreDispatcher, ObjectDispatcher):
    """
    TGController is a specialized form of ObjectDispatchController that forms the
    basis of standard TurboGears controllers.  The "Root" controller of a standard
    tg project must be a TGController.

    This controller can be used as a baseclass for anything in the
    object dispatch tree, but it MUST be used in the Root controller
    and any controller which you intend to do object dispatch.

    This controller has a few reserved method names which provide special functionality.

    +-----------------+--------------------------------------------------------------+--------------------------------------------+
    | Method          | Description                                                  | Example URL(s)                             |
    +=================+==============================================================+============================================+
    | index           | The root of the controller.                                  | /                                          |
    +-----------------+--------------------------------------------------------------+--------------------------------------------+
    | _default        | A method to call when all other methods have failed.         | /movies                                    |
    +-----------------+--------------------------------------------------------------+--------------------------------------------+
    | _lookup         | Allows the developer to return a                             | /location/23.35/2343.34/elevation          |
    |                 | Controller instance for further dispatch.                    |                                            |
    +-----------------+--------------------------------------------------------------+--------------------------------------------+


    :References:

      `Controller <../main/Controllers.html>`_  A basic overview on how to write controller methods.

    """

__all__ = ['TGController']




class NoDefault(object):
    pass


class EmptyContext(object):
    """
    Noop Python Context, does nothing but can be used
    in a with statement to provide a default context.
    """
    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass


from webtest import TestApp
from nose.tools import raises
from webob import Request
from tg.support.statics import StaticsMiddleware, FileServeApp
from webob.exc import HTTPBadRequest, HTTPForbidden
from datetime import datetime

def FakeApp(environ, start_response):
    return ['APP']

class TestStatics(object):
    def setup(self):
        self.app = TestApp(StaticsMiddleware(FakeApp, './tests'))

    def test_plain_request(self):
        r = self.app.get('/test.html')
        assert 'Welcome to TurboGears 2.0' in r

    def test_unknown_content_type(self):
        r = self.app.get('/empty_file.unknown')
        assert r.content_type == 'application/octet-stream'
        assert 'EMPTY' in r

    def test_if_modified_since(self):
        r = self.app.get('/empty_file.unknown', headers={'If-Modified-Since':'Sat, 29 Oct 1994 19:43:31 GMT'})
        assert 'EMPTY' in r

    @raises(HTTPBadRequest)
    def test_if_modified_since_invalid_date(self):
        r = self.app.get('/empty_file.unknown', headers={'If-Modified-Since':'This is not a date'})

    def test_if_modified_since_future(self):
        next_year = datetime.utcnow()
        next_year.replace(year=next_year.year+1)

        r = self.app.get('/empty_file.unknown',
                         headers={'If-Modified-Since':FileServeApp.make_date(next_year)},
                         status=304)

    def test_if_none_match(self):
        r = self.app.get('/empty_file.unknown')
        etag = r.headers['ETag']

        r = self.app.get('/empty_file.unknown', headers={'If-None-Match':etag}, status=304)

    def test_if_none_match_different(self):
        r = self.app.get('/empty_file.unknown', headers={'If-None-Match':'Probably-Not-The-Etag'})
        assert 'EMPTY' in r

    def test_make_date(self):
        res = FileServeApp.make_date(datetime(2000, 1, 1, 0, 0, 0, 0))
        assert res == 'Sat, 01 Jan 2000 00:00:00 GMT'

    def test_304_on_post(self):
        r = self.app.post('/empty_file.unknown', status=304)

    def test_forbidden_path(self):
        r = self.app.get('/missing/../test.html', status=404)
        assert 'Out of bounds' in r

    def test_FileApp_non_existing_file(self):
        fa = TestApp(FileServeApp('this_does_not_exists.unknown', 0))
        r = fa.get('/', status=403)
        assert '403' in r

    def test_wsgi_file_wrapper(self):
        class DummyWrapper(object):
            def __init__(self, file, block_size):
                self.file = file
                self.block_size = block_size

        environ = {
            'wsgi.url_scheme': 'http',
            'wsgi.version':(1,0),
            'wsgi.file_wrapper': DummyWrapper,
            'SERVER_NAME': 'somedomain.com',
            'SERVER_PORT': '8080',
            'PATH_INFO': '/index.html',
            'SCRIPT_NAME': '',
            'REQUEST_METHOD': 'GET',
            }

        app = FileServeApp('./tests/test.html', 3600)
        app_iter = Request(environ).send(app).app_iter
        assert isinstance(app_iter, DummyWrapper)
        assert b'Welcome to TurboGears 2.0' in app_iter.file.read()
        app_iter.file.close()



import warnings

from tg._compat import unicode_text

from .i18n import _formencode_gettext, lazy_ugettext

try:
    from tw2.core import ValidationError as _Tw2ValidationError
except ImportError: #pragma: no cover
    class _Tw2ValidationError(Exception):
        """ToscaWidgets2 Validation Error"""

try:
    from formencode.api import Invalid as _FormEncodeValidationError
    from formencode.api import Validator as _FormEncodeValidator
    from formencode import Schema as _FormEncodeSchema
except ImportError: #pragma: no cover
    class _FormEncodeValidationError(Exception):
        """FormEncode Invalid"""
    class _FormEncodeValidator(object):
        """FormEncode Validator"""
    class _FormEncodeSchema(object):
        """FormEncode Schema"""


class _ValidationStatus(object):
    """Current request parameters validation status.

    Keeps track of currently validated values, errors and
    the ValidationIntent that caused the validation process.
    """
    __slots__ = ('errors', 'values', 'exception', 'intent')

    def __init__(self, errors=None, values=None, exception=None, intent=None):
        self.errors = errors or {}
        self.values = values or {}
        self.exception = exception
        self.intent = intent

    @property
    def error_handler(self):
        if self.intent is None:
            return None

        return self.intent.error_handler

    def __getitem__(self, item):
        warnings.warn("Accessing validation status properties with [] syntax is deprecated. "
                      " Please use dot notation instead", DeprecationWarning, stacklevel=2)
        try:
            return getattr(self, item)
        except AttributeError:
            raise KeyError


class _ValidationIntent(object):
    """Details of validation intention.

    Describes how a validation should happen and how
    errors should be handled. It also performs the
    validation itself on the parameters for a given
    controller method.
    """
    def __init__(self, validators, error_handler):
        self.validators = validators
        self.error_handler = error_handler

    def check(self, method, params):
        validators = self.validators
        if not validators:
            return params

        # An object used by FormEncode to get translator function
        formencode_state = type('state', (), {'_': staticmethod(_formencode_gettext)})
        validated_params = {}

        # The validator may be a dictionary, a FormEncode Schema object, or any
        # object with a "validate" method.
        if isinstance(validators, dict):
            # TG developers can pass in a dict of param names and FormEncode
            # validators.  They are applied one by one and builds up a new set
            # of validated params.

            errors = {}
            for field, validator in validators.items():
                try:
                    if isinstance(validator, _FormEncodeValidator):
                        validated_params[field] = validator.to_python(params.get(field), formencode_state)
                    else:
                        validated_params[field] = validator.to_python(params.get(field))
                # catch individual validation errors into the errors dictionary
                except validation_errors as inv:
                    errors[field] = inv

            # Parameters that don't have validators are returned verbatim
            for param, param_value in params.items():
                if param not in validated_params:
                    validated_params[param] = param_value

            # If there are errors, create a compound validation error based on
            # the errors dictionary, and raise it as an exception
            if errors:
                raise TGValidationError(TGValidationError.make_compound_message(errors),
                                        value=params,
                                        error_dict=errors)

        elif isinstance(validators, _FormEncodeSchema):
            # A FormEncode Schema object - to_python converts the incoming
            # parameters to sanitized Python values
            validated_params = validators.to_python(params, formencode_state)

        elif hasattr(validators, 'validate') and getattr(self, 'needs_controller', False):
            # An object with a "validate" method - call it with the parameters
            validated_params = validators.validate(method, params, formencode_state)

        elif hasattr(validators, 'validate'):
            # An object with a "validate" method - call it with the parameters
            validated_params = validators.validate(params, formencode_state)

        return validated_params


def _navigate_tw2form_children(w):
    if getattr(w, 'compound_key', None):
        # If we have a compound_key it's a leaf widget with form values
        yield w
    else:
        child = getattr(w, 'child', None)
        if child:
            # Widgets with "child" don't have children, but their child has
            w = child

        for c in getattr(w, 'children', []):
            for cc in _navigate_tw2form_children(c):
                yield cc


class TGValidationError(Exception):
    """Invalid data was encountered during validation.

    The constructor can be passed a short message with
    the reason of the failed validation.
    """
    def __init__(self, msg, value=None, error_dict=None):
        super(TGValidationError, self).__init__(msg)
        self.msg = msg
        self.value = value
        self.error_dict = error_dict

    @classmethod
    def make_compound_message(cls, error_dict):
        return unicode_text('\n').join(
            unicode_text("%s: %s") % errorinfo for errorinfo in error_dict.items()
        )

    def __str__(self):
        return str(self.msg)

    def __unicode__(self):
        return unicode_text(self.msg)


class Convert(object):
    """Applies a conversion function as a validator.

    This is meant to implement simple validation mechanism.

    Any callable can be used for ``func`` as far as it accepts an argument and
    returns the converted object. In case of exceptions the validation
    is considered failed and the ``msg`` parameter is displayed as
    an error.

    A ``default`` value can be provided for values that are missing
    (evaluate to false) which will be used in place of the missing value.

    Example::

        @expose()
        @validate({
            'num': Convert(int, 'Must be a number')
        }, error_handler=insert_number)
        def post_pow2(self, num):
            return str(num*num)
    """
    def __init__(self, func, msg=lazy_ugettext('Invalid'), default=None):
        self._func = func
        self._msg = msg
        self._default = default

    def to_python(self, value, state=None):
        value = value or self._default

        try:
            return self._func(value)
        except:
            raise TGValidationError(self._msg, value)


validation_errors = (_Tw2ValidationError, _FormEncodeValidationError, TGValidationError)



from tg._compat import reraise
from tg.request_local import Request
import sys
import transaction

import logging
log = logging.getLogger('tg.transaction_manager')


class AbortTransaction(Exception):
    def __init__(self, response_data):
        self.response_data = response_data


class TGTransactionManager(object):
    def __init__(self, app, config):
        self.app = app
        self.config = config

        self.attempts = config.get('tm.attempts', 1)
        self.commit_veto = config.get('tm.commit_veto', None)

    def __call__(self, environ, start_response):
        if 'repoze.tm.active' in environ: #pragma: no cover
            #Skip transaction manager if repoze.tm2 is enabled
            return self.app(environ, start_response)

        transaction_manager = transaction.manager
        total_attempts = self.attempts
        commit_veto = self.commit_veto
        started_response = {}

        def _start_response(status, headers, exc_info=None):
            started_response.update(status=status, headers=headers)
            return start_response(status, headers, exc_info)

        attempts_left = total_attempts
        while attempts_left:
            attempts_left -= 1

            try:
                log.debug('Attempts Left %d (%d total)', attempts_left, total_attempts)
                transaction_manager.begin()

                if total_attempts > 1:
                    Request(environ).make_body_seekable()

                t = transaction_manager.get()
                t.note(environ.get('PATH_INFO', ''))

                response_data = self.app(environ, _start_response)
                if transaction_manager.isDoomed():
                    log.debug('Transaction doomed')
                    raise AbortTransaction(response_data)

                if commit_veto is not None:
                    veto = commit_veto(environ, started_response['status'], started_response['headers'])
                    if veto:
                        log.debug('Transaction vetoed')
                        raise AbortTransaction(response_data)

                transaction_manager.commit()
                log.debug('Transaction committed!')
                return response_data
            except AbortTransaction as e:
                transaction_manager.abort()
                return e.response_data
            except:
                exc_info = sys.exc_info()
                log.debug('Error while running request, aborting transaction')
                try:
                    can_retry = transaction_manager._retryable(*exc_info[:-1])
                    transaction_manager.abort()
                    if (attempts_left <= 0) or (not can_retry):
                        reraise(*exc_info)
                finally:
                    del exc_info



import tg

class FakePackage(object):
    pass

default_config = {
        'debug': False,
        'package': FakePackage,
        'package_name' : 'FakePackage',
        'paths': {'root': None,
                         'controllers': None,
                         'templates': [],
                         'static_files': None},
        'db_engines': {},
        'tg.strict_tmpl_context':False,
        'use_dotted_templatenames':True,
        'buffet.template_engines': [],
        'buffet.template_options': {},
        'default_renderer':'json',
        'renderers': ['json'],
        'render_functions': {'json': tg.renderers.json.JSONRenderer.render_json},
        'rendering_engines_options': {'json': {'content_type': 'application/json'}},
        'rendering_engines_without_vars': set(('json',)),
        'use_legacy_renderers': False,
        'use_sqlalchemy': False,
        'i18n.lang': None
}


class ControllerWrap(object):
    def __init__(self, controller):
        self.controller = controller

    def __call__(self, environ, start_response):
        app = self.controller()
        return app(environ, start_response)



from tg.error import ErrorReporter
from tg.error import SlowReqsReporter


def simple_app(environ, start_response):
    status = '200 OK'
    headers = [('Content-type', 'text/plain')]
    start_response(status, headers)
    return ['HELLO']


class TestErrorReporterConfig(object):

    middleware_name = 'TraceErrorsMiddleware'

    def test_enable_none(self):
        app = ErrorReporter(simple_app, {})
        assert app.__class__.__name__ == self.middleware_name
        assert not app.reporters

    def test_enable_false(self):
        app = ErrorReporter(simple_app, {}, enable=False)
        assert app.__class__.__name__ != self.middleware_name

    def test_enable_true(self):
        app = ErrorReporter(simple_app, {}, enable=True)
        assert app.__class__.__name__ == self.middleware_name
        assert not app.reporters

    def test_enable_email(self):
        app = ErrorReporter(simple_app, {},
            error_email='user@somedomain.com')
        assert app.__class__.__name__ == self.middleware_name
        assert any(r.__class__.__name__ == 'EmailReporter'
            for r in app.reporters)

    def test_enable_sentry(self):
        app = ErrorReporter(simple_app, {},
            sentry_dsn='http://public:secret@example.com/1')
        assert app.__class__.__name__ == self.middleware_name
        assert any(r.__class__.__name__ == 'SentryReporter'
            for r in app.reporters)

    def test_debug_mode(self):
        app = ErrorReporter(simple_app, dict(debug='on'), enable=True,
            error_email='user@somedomain.com')
        assert app is simple_app


class TestSlowReqsReporterConfig(object):

    middleware_name = 'TraceSlowRequestsMiddleware'

    def test_disable_all(self):
        app = SlowReqsReporter(simple_app, {})
        assert app is simple_app

    def test_enable_without_reporter(self):
        app = SlowReqsReporter(simple_app, {}, enable=True)
        assert app.__class__.__name__ == self.middleware_name
        assert not app.reporters

    def test_enable_email(self):
        app = SlowReqsReporter(simple_app, {}, enable=True,
            error_email='user@somedomain.com')
        assert app.__class__.__name__ == self.middleware_name
        assert any(r.__class__.__name__ == 'EmailReporter'
            for r in app.reporters)

    def test_enable_sentry(self):
        app = SlowReqsReporter(simple_app, {}, enable=True,
            sentry_dsn='http://public:secret@example.com/1')
        assert app.__class__.__name__ == self.middleware_name
        assert any(r.__class__.__name__ == 'SentryReporter'
            for r in app.reporters)

    def test_debug_mode(self):
        app = SlowReqsReporter(simple_app, dict(debug='on'), enable=True,
            error_email='user@somedomain.com')
        assert app is simple_app



"""TurboGears project related information"""
version = "2.3.11"
description = "Next generation TurboGears"
long_description="""
TurboGears brings together a best of breed python tools
to create a flexible, full featured, and easy to use web
framework.

TurboGears 2 provides an integrated and well tested set of tools for
everything you need to build dynamic, database driven applications.
It provides a full range of tools for front end javascript
develeopment, back database development and everything in between:

 * dynamic javascript powered widgets (ToscaWidgets2)
 * automatic JSON generation from your controllers
 * powerful, designer friendly XHTML based templating (Genshi)
 * object or route based URL dispatching
 * powerful Object Relational Mappers (SQLAlchemy)

The latest development version is available in the
`TurboGears Git repositories`_.

.. _TurboGears Git repositories:
    https://github.com/TurboGears
"""
url="http://www.turbogears.org/"
author= "Mark Ramm, Christopher Perkins, Jonathan LaCour, Rick Copland, Alberto Valverde, Michael Pedersen, Alessandro Molina, and the TurboGears community"
email = "m.pedersen@icelus.org, amol@turbogears.org"
copyright = """Copyright 2005-2017 Kevin Dangoor, Alberto Valverde, Mark Ramm, Christopher Perkins, Alessandro Molina and contributors"""
license = "MIT"
